@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2008 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>.
- * Copyright (C) 2008 Nokia Corporation, all rights reserved.
+ * Copyright (C) 2008 Nokia Corporation.
  *
  * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *                               <zeeshan.ali@nokia.com>
@@ -22,44 +22,62 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include "rygel-test-item.h"
+#include <glib.h>
+#include <glib-object.h>
+#include <rygel.h>
+#include <stdlib.h>
+#include <string.h>
 
 
+#define RYGEL_TYPE_TEST_ITEM (rygel_test_item_get_type ())
+#define RYGEL_TEST_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_TEST_ITEM, RygelTestItem))
+#define RYGEL_TEST_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_TEST_ITEM, RygelTestItemClass))
+#define RYGEL_IS_TEST_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_TEST_ITEM))
+#define RYGEL_IS_TEST_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_TEST_ITEM))
+#define RYGEL_TEST_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_TEST_ITEM, RygelTestItemClass))
+
+typedef struct _RygelTestItem RygelTestItem;
+typedef struct _RygelTestItemClass RygelTestItemClass;
+typedef struct _RygelTestItemPrivate RygelTestItemPrivate;
+#define _g_free0(var) (var = (g_free (var), NULL))
+
+struct _RygelTestItem {
+	RygelMediaItem parent_instance;
+	RygelTestItemPrivate * priv;
+};
+
+struct _RygelTestItemClass {
+	RygelMediaItemClass parent_class;
+};
 
 
+static gpointer rygel_test_item_parent_class = NULL;
+
+GType rygel_test_item_get_type (void);
 enum  {
 	RYGEL_TEST_ITEM_DUMMY_PROPERTY
 };
 #define RYGEL_TEST_ITEM_TEST_AUTHOR "Zeeshan Ali (Khattak)"
-static gpointer rygel_test_item_parent_class = NULL;
+RygelTestItem* rygel_test_item_construct (GType object_type, const char* id, RygelMediaContainer* parent, const char* title, const char* mime, const char* upnp_class);
 
 
 
 RygelTestItem* rygel_test_item_construct (GType object_type, const char* id, RygelMediaContainer* parent, const char* title, const char* mime, const char* upnp_class) {
 	RygelTestItem * self;
-	char* _tmp1;
-	const char* _tmp0;
-	char* _tmp2;
+	char* _tmp0_;
+	char* _tmp1_;
 	g_return_val_if_fail (id != NULL, NULL);
 	g_return_val_if_fail (parent != NULL, NULL);
 	g_return_val_if_fail (title != NULL, NULL);
 	g_return_val_if_fail (mime != NULL, NULL);
 	g_return_val_if_fail (upnp_class != NULL, NULL);
 	self = (RygelTestItem*) rygel_media_item_construct (object_type, id, parent, title, upnp_class);
-	_tmp1 = NULL;
-	_tmp0 = NULL;
-	((RygelMediaItem*) self)->mime_type = (_tmp1 = (_tmp0 = mime, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), ((RygelMediaItem*) self)->mime_type = (g_free (((RygelMediaItem*) self)->mime_type), NULL), _tmp1);
-	_tmp2 = NULL;
-	((RygelMediaItem*) self)->author = (_tmp2 = g_strdup (RYGEL_TEST_ITEM_TEST_AUTHOR), ((RygelMediaItem*) self)->author = (g_free (((RygelMediaItem*) self)->author), NULL), _tmp2);
+	((RygelMediaItem*) self)->mime_type = (_tmp0_ = g_strdup (mime), _g_free0 (((RygelMediaItem*) self)->mime_type), _tmp0_);
+	((RygelMediaItem*) self)->author = (_tmp1_ = g_strdup (RYGEL_TEST_ITEM_TEST_AUTHOR), _g_free0 (((RygelMediaItem*) self)->author), _tmp1_);
 	return self;
 }
 
 
-RygelTestItem* rygel_test_item_new (const char* id, RygelMediaContainer* parent, const char* title, const char* mime, const char* upnp_class) {
-	return rygel_test_item_construct (RYGEL_TYPE_TEST_ITEM, id, parent, title, mime, upnp_class);
-}
-
-
 static void rygel_test_item_class_init (RygelTestItemClass * klass) {
 	rygel_test_item_parent_class = g_type_class_peek_parent (klass);
 }
