@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Nokia Corporation, all rights reserved.
+ * Copyright (C) 2009 Nokia Corporation.
  *
  * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *                               <zeeshan.ali@nokia.com>
@@ -26,13 +26,11 @@
 #include <libgupnp-av/gupnp-av.h>
 #include <stdlib.h>
 #include <string.h>
+#include <gee.h>
 #include <gst/gst.h>
-#include <gee/arraylist.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
 #include <gio/gio.h>
-#include <gee/list.h>
-#include <gee/collection.h>
+#include <libgupnp/gupnp.h>
+#include <libgssdp/gssdp.h>
 
 
 #define RYGEL_TYPE_DIDL_LITE_WRITER (rygel_didl_lite_writer_get_type ())
@@ -65,6 +63,7 @@ typedef struct _RygelTranscodeManagerClass RygelTranscodeManagerClass;
 
 typedef struct _RygelHTTPServer RygelHTTPServer;
 typedef struct _RygelHTTPServerClass RygelHTTPServerClass;
+#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
 
 #define RYGEL_TYPE_MEDIA_OBJECT (rygel_media_object_get_type ())
 #define RYGEL_MEDIA_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_MEDIA_OBJECT, RygelMediaObject))
@@ -97,15 +96,42 @@ typedef struct _RygelMediaContainer RygelMediaContainer;
 typedef struct _RygelMediaContainerClass RygelMediaContainerClass;
 typedef struct _RygelMediaObjectPrivate RygelMediaObjectPrivate;
 typedef struct _RygelMediaItemPrivate RygelMediaItemPrivate;
+
+#define RYGEL_TYPE_ICON_INFO (rygel_icon_info_get_type ())
+#define RYGEL_ICON_INFO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_ICON_INFO, RygelIconInfo))
+#define RYGEL_ICON_INFO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_ICON_INFO, RygelIconInfoClass))
+#define RYGEL_IS_ICON_INFO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_ICON_INFO))
+#define RYGEL_IS_ICON_INFO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_ICON_INFO))
+#define RYGEL_ICON_INFO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_ICON_INFO, RygelIconInfoClass))
+
+typedef struct _RygelIconInfo RygelIconInfo;
+typedef struct _RygelIconInfoClass RygelIconInfoClass;
+
+#define RYGEL_TYPE_THUMBNAIL (rygel_thumbnail_get_type ())
+#define RYGEL_THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_THUMBNAIL, RygelThumbnail))
+#define RYGEL_THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_THUMBNAIL, RygelThumbnailClass))
+#define RYGEL_IS_THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_THUMBNAIL))
+#define RYGEL_IS_THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_THUMBNAIL))
+#define RYGEL_THUMBNAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_THUMBNAIL, RygelThumbnailClass))
+
+typedef struct _RygelThumbnail RygelThumbnail;
+typedef struct _RygelThumbnailClass RygelThumbnailClass;
+typedef struct _RygelTranscodeManagerPrivate RygelTranscodeManagerPrivate;
+
+#define RYGEL_TYPE_STATE_MACHINE (rygel_state_machine_get_type ())
+#define RYGEL_STATE_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_STATE_MACHINE, RygelStateMachine))
+#define RYGEL_IS_STATE_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_STATE_MACHINE))
+#define RYGEL_STATE_MACHINE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), RYGEL_TYPE_STATE_MACHINE, RygelStateMachineIface))
+
+typedef struct _RygelStateMachine RygelStateMachine;
+typedef struct _RygelStateMachineIface RygelStateMachineIface;
+typedef struct _RygelHTTPServerPrivate RygelHTTPServerPrivate;
 typedef struct _RygelMediaContainerPrivate RygelMediaContainerPrivate;
 
 typedef enum  {
 	RYGEL_DIDL_LITE_WRITER_ERROR_UNSUPPORTED_OBJECT
 } RygelDIDLLiteWriterError;
 #define RYGEL_DIDL_LITE_WRITER_ERROR rygel_didl_lite_writer_error_quark ()
-/**
- * Responsible for serializing media objects.
- */
 struct _RygelDIDLLiteWriter {
 	GUPnPDIDLLiteWriter parent_instance;
 	RygelDIDLLiteWriterPrivate * priv;
@@ -119,24 +145,21 @@ struct _RygelDIDLLiteWriterPrivate {
 	RygelHTTPServer* http_server;
 };
 
-/**
- * Represents a media object (container and item).
- */
 struct _RygelMediaObject {
 	GObject parent_instance;
 	RygelMediaObjectPrivate * priv;
 	char* id;
 	char* title;
+	guint64 modified;
+	GeeArrayList* uris;
 	RygelMediaContainer* parent;
+	RygelMediaContainer* parent_ref;
 };
 
 struct _RygelMediaObjectClass {
 	GObjectClass parent_class;
 };
 
-/**
- * Represents a media (Music, Video and Image) item.
- */
 struct _RygelMediaItem {
 	RygelMediaObject parent_instance;
 	RygelMediaItemPrivate * priv;
@@ -144,8 +167,8 @@ struct _RygelMediaItem {
 	char* album;
 	char* date;
 	char* upnp_class;
-	GeeArrayList* uris;
 	char* mime_type;
+	char* dlna_profile;
 	glong size;
 	glong duration;
 	gint bitrate;
@@ -155,20 +178,49 @@ struct _RygelMediaItem {
 	gint track_number;
 	gint width;
 	gint height;
+	gint pixel_width;
+	gint pixel_height;
 	gint color_depth;
+	GeeArrayList* thumbnails;
 };
 
 struct _RygelMediaItemClass {
 	RygelMediaObjectClass parent_class;
 	GstElement* (*create_stream_source) (RygelMediaItem* self);
+	gboolean (*should_stream) (RygelMediaItem* self);
+};
+
+struct _RygelTranscodeManager {
+	GObject parent_instance;
+	RygelTranscodeManagerPrivate * priv;
+};
+
+struct _RygelTranscodeManagerClass {
+	GObjectClass parent_class;
+	char* (*create_uri_for_item) (RygelTranscodeManager* self, RygelMediaItem* item, gint thumbnail_index, const char* transcode_target, char** protocol);
+	void (*add_resources) (RygelTranscodeManager* self, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, GError** error);
+	char* (*get_protocol) (RygelTranscodeManager* self);
+	char* (*get_protocol_info) (RygelTranscodeManager* self);
+};
+
+struct _RygelStateMachineIface {
+	GTypeInterface parent_iface;
+	void (*run) (RygelStateMachine* self);
+	GCancellable* (*get_cancellable) (RygelStateMachine* self);
+	void (*set_cancellable) (RygelStateMachine* self, GCancellable* value);
+};
+
+struct _RygelHTTPServer {
+	RygelTranscodeManager parent_instance;
+	RygelHTTPServerPrivate * priv;
+	RygelMediaContainer* root_container;
+	GUPnPContext* context;
+};
+
+struct _RygelHTTPServerClass {
+	RygelTranscodeManagerClass parent_class;
 };
 
-/**
- * Represents a container (folder) for media items and containers. Provides
- * basic serialization (to DIDLLiteWriter) implementation. Deriving classes
- * are supposed to provide working implementations of get_children and
- * find_object.
- */
 struct _RygelMediaContainer {
 	RygelMediaObject parent_instance;
 	RygelMediaContainerPrivate * priv;
@@ -185,6 +237,7 @@ struct _RygelMediaContainerClass {
 };
 
 
+static gpointer rygel_didl_lite_writer_parent_class = NULL;
 
 GQuark rygel_didl_lite_writer_error_quark (void);
 GType rygel_didl_lite_writer_get_type (void);
@@ -196,20 +249,24 @@ enum  {
 };
 RygelDIDLLiteWriter* rygel_didl_lite_writer_new (RygelHTTPServer* http_server);
 RygelDIDLLiteWriter* rygel_didl_lite_writer_construct (GType object_type, RygelHTTPServer* http_server);
-RygelDIDLLiteWriter* rygel_didl_lite_writer_new (RygelHTTPServer* http_server);
 GType rygel_media_object_get_type (void);
 GType rygel_media_item_get_type (void);
 static void rygel_didl_lite_writer_serialize_item (RygelDIDLLiteWriter* self, RygelMediaItem* item, GError** error);
 GType rygel_media_container_get_type (void);
 static void rygel_didl_lite_writer_serialize_container (RygelDIDLLiteWriter* self, RygelMediaContainer* container, GError** error);
 void rygel_didl_lite_writer_serialize (RygelDIDLLiteWriter* self, RygelMediaObject* media_object, GError** error);
+gpointer rygel_icon_info_ref (gpointer instance);
+void rygel_icon_info_unref (gpointer instance);
+GParamSpec* rygel_param_spec_icon_info (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void rygel_value_set_icon_info (GValue* value, gpointer v_object);
+gpointer rygel_value_get_icon_info (const GValue* value);
+GType rygel_icon_info_get_type (void);
+GType rygel_thumbnail_get_type (void);
 #define RYGEL_MEDIA_ITEM_VIDEO_CLASS "object.item.videoItem"
 #define RYGEL_MEDIA_ITEM_MUSIC_CLASS "object.item.audioItem.musicTrack"
-static GeeArrayList* rygel_didl_lite_writer_get_original_resources (RygelDIDLLiteWriter* self, RygelMediaItem* item, GError** error);
-void rygel_transcode_manager_add_resources (RygelTranscodeManager* self, GeeArrayList* resources, RygelMediaItem* item, GError** error);
-static GUPnPDIDLLiteResource* _gupnp_didl_lite_resource_dup (GUPnPDIDLLiteResource* self);
-GUPnPDIDLLiteResource rygel_media_item_create_res (RygelMediaItem* self, const char* uri, GError** error);
-static gpointer rygel_didl_lite_writer_parent_class = NULL;
+void rygel_transcode_manager_add_resources (RygelTranscodeManager* self, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, GError** error);
+GType rygel_state_machine_get_type (void);
+void rygel_media_item_add_resources (RygelMediaItem* self, GUPnPDIDLLiteItem* didl_item, gboolean allow_internal, GError** error);
 static void rygel_didl_lite_writer_finalize (GObject* obj);
 static int _vala_strcmp0 (const char * str1, const char * str2);
 
@@ -220,15 +277,17 @@ GQuark rygel_didl_lite_writer_error_quark (void) {
 }
 
 
+static gpointer _g_object_ref0 (gpointer self) {
+	return self ? g_object_ref (self) : NULL;
+}
+
+
 RygelDIDLLiteWriter* rygel_didl_lite_writer_construct (GType object_type, RygelHTTPServer* http_server) {
 	RygelDIDLLiteWriter * self;
-	RygelHTTPServer* _tmp1_;
 	RygelHTTPServer* _tmp0_;
 	g_return_val_if_fail (http_server != NULL, NULL);
 	self = g_object_newv (object_type, 0, NULL);
-	_tmp1_ = NULL;
-	_tmp0_ = NULL;
-	self->priv->http_server = (_tmp1_ = (_tmp0_ = http_server, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->http_server == NULL) ? NULL : (self->priv->http_server = (g_object_unref (self->priv->http_server), NULL)), _tmp1_);
+	self->priv->http_server = (_tmp0_ = _g_object_ref0 (http_server), _g_object_unref0 (self->priv->http_server), _tmp0_);
 	return self;
 }
 
@@ -269,17 +328,25 @@ void rygel_didl_lite_writer_serialize (RygelDIDLLiteWriter* self, RygelMediaObje
 
 static void rygel_didl_lite_writer_serialize_item (RygelDIDLLiteWriter* self, RygelMediaItem* item, GError** error) {
 	GError * _inner_error_;
+	GUPnPDIDLLiteItem* didl_item;
 	gboolean _tmp0_;
 	gboolean _tmp1_;
 	gboolean _tmp2_;
-	GeeArrayList* resources;
+	gboolean _tmp3_;
+	gboolean internal_allowed;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (item != NULL);
 	_inner_error_ = NULL;
-	gupnp_didl_lite_writer_start_item ((GUPnPDIDLLiteWriter*) self, ((RygelMediaObject*) item)->id, ((RygelMediaObject*) ((RygelMediaObject*) item)->parent)->id, NULL, FALSE);
-	/* Add fields */
-	gupnp_didl_lite_writer_add_string ((GUPnPDIDLLiteWriter*) self, "title", GUPNP_DIDL_LITE_WRITER_NAMESPACE_DC, NULL, ((RygelMediaObject*) item)->title);
-	gupnp_didl_lite_writer_add_string ((GUPnPDIDLLiteWriter*) self, "class", GUPNP_DIDL_LITE_WRITER_NAMESPACE_UPNP, NULL, item->upnp_class);
+	didl_item = gupnp_didl_lite_writer_add_item ((GUPnPDIDLLiteWriter*) self);
+	gupnp_didl_lite_object_set_id ((GUPnPDIDLLiteObject*) didl_item, ((RygelMediaObject*) item)->id);
+	if (((RygelMediaObject*) item)->parent != NULL) {
+		gupnp_didl_lite_object_set_parent_id ((GUPnPDIDLLiteObject*) didl_item, ((RygelMediaObject*) ((RygelMediaObject*) item)->parent)->id);
+	} else {
+		gupnp_didl_lite_object_set_parent_id ((GUPnPDIDLLiteObject*) didl_item, "0");
+	}
+	gupnp_didl_lite_object_set_restricted ((GUPnPDIDLLiteObject*) didl_item, FALSE);
+	gupnp_didl_lite_object_set_title ((GUPnPDIDLLiteObject*) didl_item, ((RygelMediaObject*) item)->title);
+	gupnp_didl_lite_object_set_upnp_class ((GUPnPDIDLLiteObject*) didl_item, item->upnp_class);
 	_tmp0_ = FALSE;
 	if (item->author != NULL) {
 		_tmp0_ = _vala_strcmp0 (item->author, "") != 0;
@@ -287,17 +354,17 @@ static void rygel_didl_lite_writer_serialize_item (RygelDIDLLiteWriter* self, Ry
 		_tmp0_ = FALSE;
 	}
 	if (_tmp0_) {
-		gupnp_didl_lite_writer_add_string ((GUPnPDIDLLiteWriter*) self, "creator", GUPNP_DIDL_LITE_WRITER_NAMESPACE_DC, NULL, item->author);
+		gupnp_didl_lite_object_set_creator ((GUPnPDIDLLiteObject*) didl_item, item->author);
 		if (g_str_has_prefix (item->upnp_class, RYGEL_MEDIA_ITEM_VIDEO_CLASS)) {
-			gupnp_didl_lite_writer_add_string ((GUPnPDIDLLiteWriter*) self, "author", GUPNP_DIDL_LITE_WRITER_NAMESPACE_UPNP, NULL, item->author);
+			gupnp_didl_lite_object_set_author ((GUPnPDIDLLiteObject*) didl_item, item->author);
 		} else {
 			if (g_str_has_prefix (item->upnp_class, RYGEL_MEDIA_ITEM_MUSIC_CLASS)) {
-				gupnp_didl_lite_writer_add_string ((GUPnPDIDLLiteWriter*) self, "artist", GUPNP_DIDL_LITE_WRITER_NAMESPACE_UPNP, NULL, item->author);
+				gupnp_didl_lite_object_set_artist ((GUPnPDIDLLiteObject*) didl_item, item->author);
 			}
 		}
 	}
 	if (item->track_number >= 0) {
-		gupnp_didl_lite_writer_add_int ((GUPnPDIDLLiteWriter*) self, "originalTrackNumber", GUPNP_DIDL_LITE_WRITER_NAMESPACE_UPNP, NULL, item->track_number);
+		gupnp_didl_lite_object_set_track_number ((GUPnPDIDLLiteObject*) didl_item, item->track_number);
 	}
 	_tmp1_ = FALSE;
 	if (item->album != NULL) {
@@ -306,7 +373,7 @@ static void rygel_didl_lite_writer_serialize_item (RygelDIDLLiteWriter* self, Ry
 		_tmp1_ = FALSE;
 	}
 	if (_tmp1_) {
-		gupnp_didl_lite_writer_add_string ((GUPnPDIDLLiteWriter*) self, "album", GUPNP_DIDL_LITE_WRITER_NAMESPACE_UPNP, NULL, item->album);
+		gupnp_didl_lite_object_set_album ((GUPnPDIDLLiteObject*) didl_item, item->album);
 	}
 	_tmp2_ = FALSE;
 	if (item->date != NULL) {
@@ -315,104 +382,48 @@ static void rygel_didl_lite_writer_serialize_item (RygelDIDLLiteWriter* self, Ry
 		_tmp2_ = FALSE;
 	}
 	if (_tmp2_) {
-		gupnp_didl_lite_writer_add_string ((GUPnPDIDLLiteWriter*) self, "date", GUPNP_DIDL_LITE_WRITER_NAMESPACE_DC, NULL, item->date);
+		gupnp_didl_lite_object_set_date ((GUPnPDIDLLiteObject*) didl_item, item->date);
 	}
-	/* Add resource data */
-	resources = rygel_didl_lite_writer_get_original_resources (self, item, &_inner_error_);
+	rygel_transcode_manager_add_resources ((RygelTranscodeManager*) self->priv->http_server, didl_item, item, &_inner_error_);
 	if (_inner_error_ != NULL) {
 		g_propagate_error (error, _inner_error_);
+		_g_object_unref0 (didl_item);
 		return;
 	}
-	/* Now get the transcoded/proxy URIs */
-	rygel_transcode_manager_add_resources ((RygelTranscodeManager*) self->priv->http_server, resources, item, &_inner_error_);
+	_tmp3_ = FALSE;
+	if (_vala_strcmp0 (gssdp_client_get_interface ((GSSDPClient*) self->priv->http_server->context), "lo") == 0) {
+		_tmp3_ = TRUE;
+	} else {
+		_tmp3_ = _vala_strcmp0 (gssdp_client_get_host_ip ((GSSDPClient*) self->priv->http_server->context), "127.0.0.1") == 0;
+	}
+	internal_allowed = _tmp3_;
+	rygel_media_item_add_resources (item, didl_item, internal_allowed, &_inner_error_);
 	if (_inner_error_ != NULL) {
 		g_propagate_error (error, _inner_error_);
-		(resources == NULL) ? NULL : (resources = (g_object_unref (resources), NULL));
+		_g_object_unref0 (didl_item);
 		return;
 	}
-	{
-		GeeIterator* _res_it;
-		_res_it = gee_iterable_iterator ((GeeIterable*) resources);
-		while (gee_iterator_next (_res_it)) {
-			GUPnPDIDLLiteResource _tmp4_ = {0};
-			GUPnPDIDLLiteResource* _tmp3_;
-			GUPnPDIDLLiteResource _tmp5_ = {0};
-			GUPnPDIDLLiteResource res;
-			_tmp3_ = NULL;
-			res = (_tmp5_ = (gupnp_didl_lite_resource_copy (&(*(_tmp3_ = (GUPnPDIDLLiteResource*) gee_iterator_get (_res_it))), &_tmp4_), _tmp4_), (_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_free (_tmp3_), NULL)), _tmp5_);
-			gupnp_didl_lite_writer_add_res ((GUPnPDIDLLiteWriter*) self, &res);
-			gupnp_didl_lite_resource_destroy (&res);
-		}
-		(_res_it == NULL) ? NULL : (_res_it = (g_object_unref (_res_it), NULL));
-	}
-	/* End of item */
-	gupnp_didl_lite_writer_end_item ((GUPnPDIDLLiteWriter*) self);
-	(resources == NULL) ? NULL : (resources = (g_object_unref (resources), NULL));
+	_g_object_unref0 (didl_item);
 }
 
 
 static void rygel_didl_lite_writer_serialize_container (RygelDIDLLiteWriter* self, RygelMediaContainer* container, GError** error) {
-	char* parent_id;
+	GUPnPDIDLLiteContainer* didl_container;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (container != NULL);
-	parent_id = NULL;
+	didl_container = gupnp_didl_lite_writer_add_container ((GUPnPDIDLLiteWriter*) self);
 	if (((RygelMediaObject*) container)->parent != NULL) {
-		char* _tmp1_;
-		const char* _tmp0_;
-		_tmp1_ = NULL;
-		_tmp0_ = NULL;
-		parent_id = (_tmp1_ = (_tmp0_ = ((RygelMediaObject*) ((RygelMediaObject*) container)->parent)->id, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), parent_id = (g_free (parent_id), NULL), _tmp1_);
+		gupnp_didl_lite_object_set_parent_id ((GUPnPDIDLLiteObject*) didl_container, ((RygelMediaObject*) ((RygelMediaObject*) container)->parent)->id);
 	} else {
-		char* _tmp2_;
-		_tmp2_ = NULL;
-		parent_id = (_tmp2_ = g_strdup ("-1"), parent_id = (g_free (parent_id), NULL), _tmp2_);
+		gupnp_didl_lite_object_set_parent_id ((GUPnPDIDLLiteObject*) didl_container, "-1");
 	}
-	gupnp_didl_lite_writer_start_container ((GUPnPDIDLLiteWriter*) self, ((RygelMediaObject*) container)->id, parent_id, (gint) container->child_count, FALSE, FALSE);
-	gupnp_didl_lite_writer_add_string ((GUPnPDIDLLiteWriter*) self, "class", GUPNP_DIDL_LITE_WRITER_NAMESPACE_UPNP, NULL, "object.container.storageFolder");
-	gupnp_didl_lite_writer_add_string ((GUPnPDIDLLiteWriter*) self, "title", GUPNP_DIDL_LITE_WRITER_NAMESPACE_DC, NULL, ((RygelMediaObject*) container)->title);
-	/* End of Container */
-	gupnp_didl_lite_writer_end_container ((GUPnPDIDLLiteWriter*) self);
-	parent_id = (g_free (parent_id), NULL);
-}
-
-
-static GUPnPDIDLLiteResource* _gupnp_didl_lite_resource_dup (GUPnPDIDLLiteResource* self) {
-	GUPnPDIDLLiteResource* dup;
-	dup = g_new0 (GUPnPDIDLLiteResource, 1);
-	gupnp_didl_lite_resource_copy (self, dup);
-	return dup;
-}
-
-
-static GeeArrayList* rygel_didl_lite_writer_get_original_resources (RygelDIDLLiteWriter* self, RygelMediaItem* item, GError** error) {
-	GError * _inner_error_;
-	GeeArrayList* resources;
-	g_return_val_if_fail (self != NULL, NULL);
-	g_return_val_if_fail (item != NULL, NULL);
-	_inner_error_ = NULL;
-	resources = gee_array_list_new (GUPNP_TYPE_DIDL_LITE_RESOURCE, (GBoxedCopyFunc) _gupnp_didl_lite_resource_dup, g_free, g_direct_equal);
-	{
-		GeeIterator* _uri_it;
-		_uri_it = gee_iterable_iterator ((GeeIterable*) item->uris);
-		while (gee_iterator_next (_uri_it)) {
-			char* uri;
-			GUPnPDIDLLiteResource res;
-			uri = (char*) gee_iterator_get (_uri_it);
-			res = rygel_media_item_create_res (item, uri, &_inner_error_);
-			if (_inner_error_ != NULL) {
-				g_propagate_error (error, _inner_error_);
-				uri = (g_free (uri), NULL);
-				(_uri_it == NULL) ? NULL : (_uri_it = (g_object_unref (_uri_it), NULL));
-				(resources == NULL) ? NULL : (resources = (g_object_unref (resources), NULL));
-				return NULL;
-			}
-			gee_collection_add ((GeeCollection*) resources, &res);
-			uri = (g_free (uri), NULL);
-			gupnp_didl_lite_resource_destroy (&res);
-		}
-		(_uri_it == NULL) ? NULL : (_uri_it = (g_object_unref (_uri_it), NULL));
-	}
-	return resources;
+	gupnp_didl_lite_object_set_id ((GUPnPDIDLLiteObject*) didl_container, ((RygelMediaObject*) container)->id);
+	gupnp_didl_lite_object_set_title ((GUPnPDIDLLiteObject*) didl_container, ((RygelMediaObject*) container)->title);
+	gupnp_didl_lite_container_set_child_count (didl_container, container->child_count);
+	gupnp_didl_lite_object_set_restricted ((GUPnPDIDLLiteObject*) didl_container, FALSE);
+	gupnp_didl_lite_container_set_searchable (didl_container, FALSE);
+	gupnp_didl_lite_object_set_upnp_class ((GUPnPDIDLLiteObject*) didl_container, "object.container.storageFolder");
+	_g_object_unref0 (didl_container);
 }
 
 
@@ -431,7 +442,7 @@ static void rygel_didl_lite_writer_instance_init (RygelDIDLLiteWriter * self) {
 static void rygel_didl_lite_writer_finalize (GObject* obj) {
 	RygelDIDLLiteWriter * self;
 	self = RYGEL_DIDL_LITE_WRITER (obj);
-	(self->priv->http_server == NULL) ? NULL : (self->priv->http_server = (g_object_unref (self->priv->http_server), NULL));
+	_g_object_unref0 (self->priv->http_server);
 	G_OBJECT_CLASS (rygel_didl_lite_writer_parent_class)->finalize (obj);
 }
 
