@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2008 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>.
- * Copyright (C) 2008 Nokia Corporation, all rights reserved.
+ * Copyright (C) 2008 Nokia Corporation.
  *
  * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *                               <zeeshan.ali@nokia.com>
@@ -61,29 +61,46 @@ typedef struct _RygelLiveResponse RygelLiveResponse;
 typedef struct _RygelLiveResponseClass RygelLiveResponseClass;
 typedef struct _RygelLiveResponsePrivate RygelLiveResponsePrivate;
 
+#define RYGEL_TYPE_HTTP_SEEK (rygel_http_seek_get_type ())
+#define RYGEL_HTTP_SEEK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_HTTP_SEEK, RygelHTTPSeek))
+#define RYGEL_HTTP_SEEK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_HTTP_SEEK, RygelHTTPSeekClass))
+#define RYGEL_IS_HTTP_SEEK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_HTTP_SEEK))
+#define RYGEL_IS_HTTP_SEEK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_HTTP_SEEK))
+#define RYGEL_HTTP_SEEK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_HTTP_SEEK, RygelHTTPSeekClass))
+
+typedef struct _RygelHTTPSeek RygelHTTPSeek;
+typedef struct _RygelHTTPSeekClass RygelHTTPSeekClass;
+#define _gst_object_unref0(var) ((var == NULL) ? NULL : (var = (gst_object_unref (var), NULL)))
+#define _g_async_queue_unref0(var) ((var == NULL) ? NULL : (var = (g_async_queue_unref (var), NULL)))
+#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
+#define _gst_buffer_unref0(var) ((var == NULL) ? NULL : (var = (gst_buffer_unref (var), NULL)))
+#define _gst_caps_unref0(var) ((var == NULL) ? NULL : (var = (gst_caps_unref (var), NULL)))
+#define _gst_structure_free0(var) ((var == NULL) ? NULL : (var = (gst_structure_free (var), NULL)))
+#define __g_list_free_gst_object_unref0(var) ((var == NULL) ? NULL : (var = (_g_list_free_gst_object_unref (var), NULL)))
+#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
+#define _g_free0(var) (var = (g_free (var), NULL))
+
 typedef enum  {
 	RYGEL_LIVE_RESPONSE_ERROR_MISSING_PLUGIN,
 	RYGEL_LIVE_RESPONSE_ERROR_LINK
 } RygelLiveResponseError;
 #define RYGEL_LIVE_RESPONSE_ERROR rygel_live_response_error_quark ()
-/**
- * StateMachine interface.
- */
 struct _RygelStateMachineIface {
 	GTypeInterface parent_iface;
-	void (*run) (RygelStateMachine* self, GCancellable* cancellable);
+	void (*run) (RygelStateMachine* self);
+	GCancellable* (*get_cancellable) (RygelStateMachine* self);
+	void (*set_cancellable) (RygelStateMachine* self, GCancellable* value);
 };
 
 struct _RygelHTTPResponse {
 	GObject parent_instance;
 	RygelHTTPResponsePrivate * priv;
 	SoupMessage* msg;
-	GCancellable* cancellable;
 };
 
 struct _RygelHTTPResponseClass {
 	GObjectClass parent_class;
-	void (*run) (RygelHTTPResponse* self, GCancellable* cancellable);
+	void (*run) (RygelHTTPResponse* self);
 	void (*end) (RygelHTTPResponse* self, gboolean aborted, guint status);
 };
 
@@ -99,30 +116,32 @@ struct _RygelLiveResponseClass {
 struct _RygelLiveResponsePrivate {
 	GstPipeline* pipeline;
 	GAsyncQueue* buffers;
+	RygelHTTPSeek* time_range;
 };
 
 
+static gpointer rygel_live_response_parent_class = NULL;
 
 GQuark rygel_live_response_error_quark (void);
 GType rygel_state_machine_get_type (void);
 GType rygel_http_response_get_type (void);
 GType rygel_live_response_get_type (void);
+GType rygel_http_seek_get_type (void);
 #define RYGEL_LIVE_RESPONSE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RYGEL_TYPE_LIVE_RESPONSE, RygelLiveResponsePrivate))
 enum  {
 	RYGEL_LIVE_RESPONSE_DUMMY_PROPERTY
 };
 #define RYGEL_LIVE_RESPONSE_SINK_NAME "fakesink"
-RygelHTTPResponse* rygel_http_response_new (SoupServer* server, SoupMessage* msg, gboolean partial);
-RygelHTTPResponse* rygel_http_response_construct (GType object_type, SoupServer* server, SoupMessage* msg, gboolean partial);
+RygelHTTPResponse* rygel_http_response_construct (GType object_type, SoupServer* server, SoupMessage* msg, gboolean partial, GCancellable* cancellable);
 static void rygel_live_response_prepare_pipeline (RygelLiveResponse* self, const char* name, GstElement* src, GError** error);
-RygelLiveResponse* rygel_live_response_new (SoupServer* server, SoupMessage* msg, const char* name, GstElement* src, GError** error);
-RygelLiveResponse* rygel_live_response_construct (GType object_type, SoupServer* server, SoupMessage* msg, const char* name, GstElement* src, GError** error);
-RygelLiveResponse* rygel_live_response_new (SoupServer* server, SoupMessage* msg, const char* name, GstElement* src, GError** error);
-void rygel_http_response_run (RygelHTTPResponse* self, GCancellable* cancellable);
-static void rygel_live_response_real_run (RygelHTTPResponse* base, GCancellable* cancellable);
+RygelLiveResponse* rygel_live_response_new (SoupServer* server, SoupMessage* msg, const char* name, GstElement* src, RygelHTTPSeek* time_range, GCancellable* cancellable, GError** error);
+RygelLiveResponse* rygel_live_response_construct (GType object_type, SoupServer* server, SoupMessage* msg, const char* name, GstElement* src, RygelHTTPSeek* time_range, GCancellable* cancellable, GError** error);
+void rygel_http_response_run (RygelHTTPResponse* self);
+gint64 rygel_http_seek_get_start (RygelHTTPSeek* self);
+static void rygel_live_response_real_run (RygelHTTPResponse* base);
 void rygel_http_response_end (RygelHTTPResponse* self, gboolean aborted, guint status);
 static void rygel_live_response_real_end (RygelHTTPResponse* base, gboolean aborted, guint status);
-static inline void _dynamic_set_signal_handoffs1 (GstElement* obj, gboolean value);
+static inline void _dynamic_set_signal_handoffs0 (GstElement* obj, gboolean value);
 static void rygel_live_response_on_new_buffer (RygelLiveResponse* self, GstElement* sink, GstBuffer* buffer, GstPad* pad);
 static void _rygel_live_response_on_new_buffer_dynamic_handoff0_ (GstElement* _sender, GstBuffer* buffer, GstPad* pad, gpointer self);
 void _dynamic_handoff1_connect (gpointer obj, const char * signal_name, GCallback handler, gpointer data);
@@ -141,7 +160,8 @@ static gint _rygel_live_response_compare_factories_gcompare_func (void* a, void*
 static gboolean rygel_live_response_idle_handler (RygelLiveResponse* self);
 static gboolean _rygel_live_response_idle_handler_gsource_func (gpointer self);
 void rygel_http_response_push_data (RygelHTTPResponse* self, void* data, gsize length);
-static gpointer rygel_live_response_parent_class = NULL;
+static gboolean rygel_live_response_seek (RygelLiveResponse* self);
+gint64 rygel_http_seek_get_stop (RygelHTTPSeek* self);
 static void rygel_live_response_finalize (GObject* obj);
 static int _vala_strcmp0 (const char * str1, const char * str2);
 
@@ -153,39 +173,55 @@ GQuark rygel_live_response_error_quark (void) {
 }
 
 
-RygelLiveResponse* rygel_live_response_construct (GType object_type, SoupServer* server, SoupMessage* msg, const char* name, GstElement* src, GError** error) {
+static gpointer _g_object_ref0 (gpointer self) {
+	return self ? g_object_ref (self) : NULL;
+}
+
+
+RygelLiveResponse* rygel_live_response_construct (GType object_type, SoupServer* server, SoupMessage* msg, const char* name, GstElement* src, RygelHTTPSeek* time_range, GCancellable* cancellable, GError** error) {
 	GError * _inner_error_;
 	RygelLiveResponse * self;
 	GAsyncQueue* _tmp0_;
+	RygelHTTPSeek* _tmp1_;
 	g_return_val_if_fail (server != NULL, NULL);
 	g_return_val_if_fail (msg != NULL, NULL);
 	g_return_val_if_fail (name != NULL, NULL);
 	g_return_val_if_fail (src != NULL, NULL);
 	_inner_error_ = NULL;
-	self = (RygelLiveResponse*) rygel_http_response_construct (object_type, server, msg, FALSE);
+	self = (RygelLiveResponse*) rygel_http_response_construct (object_type, server, msg, FALSE, cancellable);
 	soup_message_headers_set_encoding (((RygelHTTPResponse*) self)->msg->response_headers, SOUP_ENCODING_EOF);
-	_tmp0_ = NULL;
-	self->priv->buffers = (_tmp0_ = g_async_queue_new (), (self->priv->buffers == NULL) ? NULL : (self->priv->buffers = (g_async_queue_unref (self->priv->buffers), NULL)), _tmp0_);
+	self->priv->buffers = (_tmp0_ = g_async_queue_new (), _g_async_queue_unref0 (self->priv->buffers), _tmp0_);
 	rygel_live_response_prepare_pipeline (self, name, src, &_inner_error_);
 	if (_inner_error_ != NULL) {
 		g_propagate_error (error, _inner_error_);
 		return;
 	}
+	self->priv->time_range = (_tmp1_ = _g_object_ref0 (time_range), _g_object_unref0 (self->priv->time_range), _tmp1_);
 	return self;
 }
 
 
-RygelLiveResponse* rygel_live_response_new (SoupServer* server, SoupMessage* msg, const char* name, GstElement* src, GError** error) {
-	return rygel_live_response_construct (RYGEL_TYPE_LIVE_RESPONSE, server, msg, name, src, error);
+RygelLiveResponse* rygel_live_response_new (SoupServer* server, SoupMessage* msg, const char* name, GstElement* src, RygelHTTPSeek* time_range, GCancellable* cancellable, GError** error) {
+	return rygel_live_response_construct (RYGEL_TYPE_LIVE_RESPONSE, server, msg, name, src, time_range, cancellable, error);
 }
 
 
-static void rygel_live_response_real_run (RygelHTTPResponse* base, GCancellable* cancellable) {
+static void rygel_live_response_real_run (RygelHTTPResponse* base) {
 	RygelLiveResponse * self;
+	gboolean _tmp0_;
 	self = (RygelLiveResponse*) base;
-	RYGEL_HTTP_RESPONSE_CLASS (rygel_live_response_parent_class)->run (RYGEL_HTTP_RESPONSE (self), cancellable);
-	/* Go to PAUSED first*/
-	gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_PLAYING);
+	RYGEL_HTTP_RESPONSE_CLASS (rygel_live_response_parent_class)->run (RYGEL_HTTP_RESPONSE (self));
+	_tmp0_ = FALSE;
+	if (self->priv->time_range != NULL) {
+		_tmp0_ = rygel_http_seek_get_start (self->priv->time_range) > 0;
+	} else {
+		_tmp0_ = FALSE;
+	}
+	if (_tmp0_) {
+		gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_PAUSED);
+	} else {
+		gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_PLAYING);
+	}
 }
 
 
@@ -194,22 +230,30 @@ static void rygel_live_response_real_end (RygelHTTPResponse* base, gboolean abor
 	GstBuffer* buffer;
 	self = (RygelLiveResponse*) base;
 	gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_NULL);
-	/* Flush the queue of buffers*/
 	buffer = NULL;
-	do {
-		GstBuffer* _tmp0_;
-		_tmp0_ = NULL;
-		buffer = (_tmp0_ = (GstBuffer*) g_async_queue_try_pop (self->priv->buffers), (buffer == NULL) ? NULL : (buffer = (gst_buffer_unref (buffer), NULL)), _tmp0_);
-	} while (buffer != NULL);
+	{
+		gboolean _tmp0_;
+		_tmp0_ = TRUE;
+		while (TRUE) {
+			GstBuffer* _tmp1_;
+			if (!_tmp0_) {
+				if (!(buffer != NULL)) {
+					break;
+				}
+			}
+			_tmp0_ = FALSE;
+			buffer = (_tmp1_ = (GstBuffer*) g_async_queue_try_pop (self->priv->buffers), _gst_buffer_unref0 (buffer), _tmp1_);
+		}
+	}
 	if (!aborted) {
 		soup_message_body_complete (((RygelHTTPResponse*) self)->msg->response_body);
 	}
 	RYGEL_HTTP_RESPONSE_CLASS (rygel_live_response_parent_class)->end (RYGEL_HTTP_RESPONSE (self), aborted, status);
-	(buffer == NULL) ? NULL : (buffer = (gst_buffer_unref (buffer), NULL));
+	_gst_buffer_unref0 (buffer);
 }
 
 
-static inline void _dynamic_set_signal_handoffs1 (GstElement* obj, gboolean value) {
+static inline void _dynamic_set_signal_handoffs0 (GstElement* obj, gboolean value) {
 	g_object_set (obj, "signal-handoffs", value, NULL);
 }
 
@@ -224,6 +268,11 @@ void _dynamic_handoff1_connect (gpointer obj, const char * signal_name, GCallbac
 }
 
 
+static gpointer _gst_object_ref0 (gpointer self) {
+	return self ? gst_object_ref (self) : NULL;
+}
+
+
 static void _rygel_live_response_src_pad_added_gst_element_pad_added (GstElement* _sender, GstPad* pad, gpointer self) {
 	rygel_live_response_src_pad_added (self, _sender, pad);
 }
@@ -238,8 +287,6 @@ static void rygel_live_response_prepare_pipeline (RygelLiveResponse* self, const
 	GError * _inner_error_;
 	GstElement* sink;
 	GstPipeline* _tmp0_;
-	GstElement* _tmp2_;
-	GstElement* _tmp1_;
 	GstBus* bus;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (name != NULL);
@@ -250,37 +297,31 @@ static void rygel_live_response_prepare_pipeline (RygelLiveResponse* self, const
 		_inner_error_ = g_error_new_literal (RYGEL_LIVE_RESPONSE_ERROR, RYGEL_LIVE_RESPONSE_ERROR_MISSING_PLUGIN, "Required plugin 'fakesink' missing");
 		if (_inner_error_ != NULL) {
 			g_propagate_error (error, _inner_error_);
-			(sink == NULL) ? NULL : (sink = (gst_object_unref (sink), NULL));
+			_gst_object_unref0 (sink);
 			return;
 		}
 	}
-	_dynamic_set_signal_handoffs1 (sink, TRUE);
+	_dynamic_set_signal_handoffs0 (sink, TRUE);
 	_dynamic_handoff1_connect (sink, "handoff", (GCallback) _rygel_live_response_on_new_buffer_dynamic_handoff0_, self);
-	_tmp0_ = NULL;
-	self->priv->pipeline = (_tmp0_ = (GstPipeline*) gst_pipeline_new (name), (self->priv->pipeline == NULL) ? NULL : (self->priv->pipeline = (gst_object_unref (self->priv->pipeline), NULL)), _tmp0_);
+	self->priv->pipeline = (_tmp0_ = (GstPipeline*) gst_pipeline_new (name), _gst_object_unref0 (self->priv->pipeline), _tmp0_);
 	g_assert (self->priv->pipeline != NULL);
-	_tmp2_ = NULL;
-	_tmp1_ = NULL;
-	gst_bin_add_many ((GstBin*) self->priv->pipeline, (_tmp1_ = src, (_tmp1_ == NULL) ? NULL : gst_object_ref (_tmp1_)), (_tmp2_ = sink, (_tmp2_ == NULL) ? NULL : gst_object_ref (_tmp2_)), NULL);
+	gst_bin_add_many ((GstBin*) self->priv->pipeline, _gst_object_ref0 (src), _gst_object_ref0 (sink), NULL);
 	if (src->numpads == 0) {
-		/* Seems source uses dynamic pads, link when pad available*/
 		g_signal_connect_object (src, "pad-added", (GCallback) _rygel_live_response_src_pad_added_gst_element_pad_added, self, 0);
 	} else {
-		/* static pads? easy!*/
 		if (!gst_element_link (src, sink)) {
 			_inner_error_ = g_error_new (RYGEL_LIVE_RESPONSE_ERROR, RYGEL_LIVE_RESPONSE_ERROR_LINK, "Failed to link %s to %s", gst_object_get_name ((GstObject*) src), gst_object_get_name ((GstObject*) sink));
 			if (_inner_error_ != NULL) {
 				g_propagate_error (error, _inner_error_);
-				(sink == NULL) ? NULL : (sink = (gst_object_unref (sink), NULL));
+				_gst_object_unref0 (sink);
 				return;
 			}
 		}
 	}
-	/* Bus handler*/
 	bus = gst_pipeline_get_bus (self->priv->pipeline);
 	gst_bus_add_watch (bus, _rygel_live_response_bus_handler_gst_bus_func, self);
-	(sink == NULL) ? NULL : (sink = (gst_object_unref (sink), NULL));
-	(bus == NULL) ? NULL : (bus = (gst_object_unref (bus), NULL));
+	_gst_object_unref0 (sink);
+	_gst_object_unref0 (bus);
 }
 
 
@@ -297,58 +338,55 @@ static void rygel_live_response_src_pad_added (RygelLiveResponse* self, GstEleme
 	sink_pad = NULL;
 	depay = rygel_live_response_get_rtp_depayloader (self, caps);
 	if (depay != NULL) {
-		GstElement* _tmp0_;
-		GstPad* _tmp2_;
-		GstPad* _tmp1_;
-		_tmp0_ = NULL;
-		gst_bin_add ((GstBin*) self->priv->pipeline, (_tmp0_ = depay, (_tmp0_ == NULL) ? NULL : gst_object_ref (_tmp0_)));
+		GstPad* _tmp0_;
+		gst_bin_add ((GstBin*) self->priv->pipeline, _gst_object_ref0 (depay));
 		if (!gst_element_link (depay, sink)) {
-			g_critical ("rygel-live-response.vala:122: Failed to link %s to %s", gst_object_get_name ((GstObject*) depay), gst_object_get_name ((GstObject*) sink));
+			g_critical ("rygel-live-response.vala:130: Failed to link %s to %s", gst_object_get_name ((GstObject*) depay), gst_object_get_name ((GstObject*) sink));
 			rygel_http_response_end ((RygelHTTPResponse*) self, FALSE, (guint) SOUP_STATUS_NONE);
-			(caps == NULL) ? NULL : (caps = (gst_caps_unref (caps), NULL));
-			(sink == NULL) ? NULL : (sink = (gst_object_unref (sink), NULL));
-			(sink_pad == NULL) ? NULL : (sink_pad = (gst_object_unref (sink_pad), NULL));
-			(depay == NULL) ? NULL : (depay = (gst_object_unref (depay), NULL));
+			_gst_caps_unref0 (caps);
+			_gst_object_unref0 (sink);
+			_gst_object_unref0 (sink_pad);
+			_gst_object_unref0 (depay);
 			return;
 		}
-		_tmp2_ = NULL;
-		_tmp1_ = NULL;
-		sink_pad = (_tmp2_ = (_tmp1_ = gst_element_get_compatible_pad (depay, src_pad, caps), (_tmp1_ == NULL) ? NULL : gst_object_ref (_tmp1_)), (sink_pad == NULL) ? NULL : (sink_pad = (gst_object_unref (sink_pad), NULL)), _tmp2_);
+		sink_pad = (_tmp0_ = _gst_object_ref0 (gst_element_get_compatible_pad (depay, src_pad, caps)), _gst_object_unref0 (sink_pad), _tmp0_);
 	} else {
-		GstPad* _tmp4_;
-		GstPad* _tmp3_;
-		_tmp4_ = NULL;
-		_tmp3_ = NULL;
-		sink_pad = (_tmp4_ = (_tmp3_ = gst_element_get_compatible_pad (sink, src_pad, caps), (_tmp3_ == NULL) ? NULL : gst_object_ref (_tmp3_)), (sink_pad == NULL) ? NULL : (sink_pad = (gst_object_unref (sink_pad), NULL)), _tmp4_);
+		GstPad* _tmp1_;
+		sink_pad = (_tmp1_ = _gst_object_ref0 (gst_element_get_compatible_pad (sink, src_pad, caps)), _gst_object_unref0 (sink_pad), _tmp1_);
 	}
 	if (gst_pad_link (src_pad, sink_pad) != GST_PAD_LINK_OK) {
-		g_critical ("rygel-live-response.vala:135: Failed to link pad %s to %s", gst_object_get_name ((GstObject*) src_pad), gst_object_get_name ((GstObject*) sink_pad));
+		g_critical ("rygel-live-response.vala:143: Failed to link pad %s to %s", gst_object_get_name ((GstObject*) src_pad), gst_object_get_name ((GstObject*) sink_pad));
 		rygel_http_response_end ((RygelHTTPResponse*) self, FALSE, (guint) SOUP_STATUS_NONE);
-		(caps == NULL) ? NULL : (caps = (gst_caps_unref (caps), NULL));
-		(sink == NULL) ? NULL : (sink = (gst_object_unref (sink), NULL));
-		(sink_pad == NULL) ? NULL : (sink_pad = (gst_object_unref (sink_pad), NULL));
-		(depay == NULL) ? NULL : (depay = (gst_object_unref (depay), NULL));
+		_gst_caps_unref0 (caps);
+		_gst_object_unref0 (sink);
+		_gst_object_unref0 (sink_pad);
+		_gst_object_unref0 (depay);
 		return;
 	}
 	if (depay != NULL) {
 		gst_element_sync_state_with_parent (depay);
 	}
-	(caps == NULL) ? NULL : (caps = (gst_caps_unref (caps), NULL));
-	(sink == NULL) ? NULL : (sink = (gst_object_unref (sink), NULL));
-	(sink_pad == NULL) ? NULL : (sink_pad = (gst_object_unref (sink_pad), NULL));
-	(depay == NULL) ? NULL : (depay = (gst_object_unref (depay), NULL));
+	_gst_caps_unref0 (caps);
+	_gst_object_unref0 (sink);
+	_gst_object_unref0 (sink_pad);
+	_gst_object_unref0 (depay);
+}
+
+
+static gpointer _gst_structure_copy0 (gpointer self) {
+	return self ? gst_structure_copy (self) : NULL;
 }
 
 
 static gboolean rygel_live_response_need_rtp_depayloader (RygelLiveResponse* self, GstCaps* caps) {
-	const GstStructure* _tmp0_;
+	gboolean result;
 	GstStructure* structure;
-	gboolean _tmp1_;
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (caps != NULL, FALSE);
-	_tmp0_ = NULL;
-	structure = (_tmp0_ = gst_caps_get_structure (caps, (guint) 0), (_tmp0_ == NULL) ? NULL : gst_structure_copy (_tmp0_));
-	return (_tmp1_ = _vala_strcmp0 (gst_structure_get_name (structure), "application/x-rtp") == 0, (structure == NULL) ? NULL : (structure = (gst_structure_free (structure), NULL)), _tmp1_);
+	structure = _gst_structure_copy0 (gst_caps_get_structure (caps, (guint) 0));
+	result = _vala_strcmp0 (gst_structure_get_name (structure), "application/x-rtp") == 0;
+	_gst_structure_free0 (structure);
+	return result;
 }
 
 
@@ -364,18 +402,20 @@ static void _g_list_free_gst_object_unref (GList* self) {
 
 
 static GstElement* rygel_live_response_get_rtp_depayloader (RygelLiveResponse* self, GstCaps* caps) {
+	GstElement* result;
 	GstRegistry* registry;
 	GList* features;
-	GstElement* _tmp1_;
 	g_return_val_if_fail (self != NULL, NULL);
 	g_return_val_if_fail (caps != NULL, NULL);
 	if (!rygel_live_response_need_rtp_depayloader (self, caps)) {
-		return NULL;
+		result = NULL;
+		return result;
 	}
 	registry = gst_registry_get_default ();
 	features = gst_registry_feature_filter (registry, _rygel_live_response_rtp_depay_filter_gst_plugin_feature_filter, FALSE, self);
-	_tmp1_ = NULL;
-	return (_tmp1_ = rygel_live_response_get_best_depay (self, features, caps), (features == NULL) ? NULL : (features = (_g_list_free_gst_object_unref (features), NULL)), _tmp1_);
+	result = rygel_live_response_get_best_depay (self, features, caps);
+	__g_list_free_gst_object_unref0 (features);
+	return result;
 }
 
 
@@ -385,87 +425,84 @@ static gint _rygel_live_response_compare_factories_gcompare_func (void* a, void*
 
 
 static GstElement* rygel_live_response_get_best_depay (RygelLiveResponse* self, GList* features, GstCaps* caps) {
+	GstElement* result;
 	GList* relevant_factories;
-	GstElementFactory* _tmp4_;
 	GstElementFactory* factory;
-	GstElement* _tmp5_;
 	g_return_val_if_fail (self != NULL, NULL);
 	g_return_val_if_fail (caps != NULL, NULL);
 	relevant_factories = NULL;
-	/* First construct a list of relevant factories*/
 	{
 		GList* feature_collection;
 		GList* feature_it;
 		feature_collection = features;
 		for (feature_it = feature_collection; feature_it != NULL; feature_it = feature_it->next) {
-			GstPluginFeature* _tmp2_;
 			GstPluginFeature* feature;
-			_tmp2_ = NULL;
-			feature = (_tmp2_ = (GstPluginFeature*) feature_it->data, (_tmp2_ == NULL) ? NULL : gst_object_ref (_tmp2_));
+			feature = _gst_object_ref0 ((GstPluginFeature*) feature_it->data);
 			{
-				GstElementFactory* _tmp0_;
 				GstElementFactory* factory;
-				_tmp0_ = NULL;
-				factory = (_tmp0_ = GST_ELEMENT_FACTORY (feature), (_tmp0_ == NULL) ? NULL : gst_object_ref (_tmp0_));
+				factory = _gst_object_ref0 (GST_ELEMENT_FACTORY (feature));
 				if (gst_element_factory_can_sink_caps (factory, caps)) {
-					GstElementFactory* _tmp1_;
-					_tmp1_ = NULL;
-					relevant_factories = g_list_append (relevant_factories, (_tmp1_ = factory, (_tmp1_ == NULL) ? NULL : gst_object_ref (_tmp1_)));
+					relevant_factories = g_list_append (relevant_factories, _gst_object_ref0 (factory));
 				}
-				(feature == NULL) ? NULL : (feature = (gst_object_unref (feature), NULL));
-				(factory == NULL) ? NULL : (factory = (gst_object_unref (factory), NULL));
+				_gst_object_unref0 (feature);
+				_gst_object_unref0 (factory);
 			}
 		}
 	}
 	if (g_list_length (relevant_factories) == 0) {
-		GstElement* _tmp3_;
-		/* No relevant factory available, hence no depayloader*/
-		_tmp3_ = NULL;
-		return (_tmp3_ = NULL, (relevant_factories == NULL) ? NULL : (relevant_factories = (_g_list_free_gst_object_unref (relevant_factories), NULL)), _tmp3_);
+		result = NULL;
+		__g_list_free_gst_object_unref0 (relevant_factories);
+		return result;
 	}
-	/* Then sort the list through their ranks*/
 	relevant_factories = g_list_sort (relevant_factories, _rygel_live_response_compare_factories_gcompare_func);
-	/* create an element of the top ranking factory and return it*/
-	_tmp4_ = NULL;
-	factory = (_tmp4_ = (GstElementFactory*) relevant_factories->data, (_tmp4_ == NULL) ? NULL : gst_object_ref (_tmp4_));
-	_tmp5_ = NULL;
-	return (_tmp5_ = gst_element_factory_make (gst_plugin_feature_get_name ((GstPluginFeature*) factory), NULL), (relevant_factories == NULL) ? NULL : (relevant_factories = (_g_list_free_gst_object_unref (relevant_factories), NULL)), (factory == NULL) ? NULL : (factory = (gst_object_unref (factory), NULL)), _tmp5_);
+	factory = _gst_object_ref0 ((GstElementFactory*) relevant_factories->data);
+	result = gst_element_factory_make (gst_plugin_feature_get_name ((GstPluginFeature*) factory), NULL);
+	__g_list_free_gst_object_unref0 (relevant_factories);
+	_gst_object_unref0 (factory);
+	return result;
 }
 
 
 static gboolean string_contains (const char* self, const char* needle) {
+	gboolean result;
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (needle != NULL, FALSE);
-	return strstr (self, needle) != NULL;
+	result = strstr (self, needle) != NULL;
+	return result;
 }
 
 
 static gboolean rygel_live_response_rtp_depay_filter (RygelLiveResponse* self, GstPluginFeature* feature) {
-	GstElementFactory* _tmp1_;
+	gboolean result;
 	GstElementFactory* factory;
-	gboolean _tmp2_;
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (feature != NULL, FALSE);
 	if (!g_type_is_a (G_TYPE_FROM_INSTANCE ((GObject*) feature), GST_TYPE_ELEMENT_FACTORY)) {
-		return FALSE;
+		result = FALSE;
+		return result;
 	}
-	_tmp1_ = NULL;
-	factory = (_tmp1_ = GST_ELEMENT_FACTORY (feature), (_tmp1_ == NULL) ? NULL : gst_object_ref (_tmp1_));
-	return (_tmp2_ = string_contains (gst_element_factory_get_klass (factory), "Depayloader"), (factory == NULL) ? NULL : (factory = (gst_object_unref (factory), NULL)), _tmp2_);
+	factory = _gst_object_ref0 (GST_ELEMENT_FACTORY (feature));
+	result = string_contains (gst_element_factory_get_klass (factory), "Depayloader");
+	_gst_object_unref0 (factory);
+	return result;
 }
 
 
 static gint rygel_live_response_compare_factories (void* a, void* b) {
-	GstElementFactory* _tmp0_;
+	gint result;
 	GstElementFactory* factory_a;
-	GstElementFactory* _tmp1_;
 	GstElementFactory* factory_b;
-	gint _tmp2_;
-	_tmp0_ = NULL;
-	factory_a = (_tmp0_ = GST_ELEMENT_FACTORY (a), (_tmp0_ == NULL) ? NULL : gst_object_ref (_tmp0_));
-	_tmp1_ = NULL;
-	factory_b = (_tmp1_ = GST_ELEMENT_FACTORY (b), (_tmp1_ == NULL) ? NULL : gst_object_ref (_tmp1_));
-	return (_tmp2_ = (gint) (gst_plugin_feature_get_rank ((GstPluginFeature*) factory_b) - gst_plugin_feature_get_rank ((GstPluginFeature*) factory_a)), (factory_a == NULL) ? NULL : (factory_a = (gst_object_unref (factory_a), NULL)), (factory_b == NULL) ? NULL : (factory_b = (gst_object_unref (factory_b), NULL)), _tmp2_);
+	factory_a = _gst_object_ref0 (GST_ELEMENT_FACTORY (a));
+	factory_b = _gst_object_ref0 (GST_ELEMENT_FACTORY (b));
+	result = (gint) (gst_plugin_feature_get_rank ((GstPluginFeature*) factory_b) - gst_plugin_feature_get_rank ((GstPluginFeature*) factory_a));
+	_gst_object_unref0 (factory_a);
+	_gst_object_unref0 (factory_b);
+	return result;
+}
+
+
+static gpointer _gst_buffer_ref0 (gpointer self) {
+	return self ? gst_buffer_ref (self) : NULL;
 }
 
 
@@ -475,30 +512,31 @@ static gboolean _rygel_live_response_idle_handler_gsource_func (gpointer self) {
 
 
 static void rygel_live_response_on_new_buffer (RygelLiveResponse* self, GstElement* sink, GstBuffer* buffer, GstPad* pad) {
-	GstBuffer* _tmp0_;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (sink != NULL);
 	g_return_if_fail (buffer != NULL);
 	g_return_if_fail (pad != NULL);
-	_tmp0_ = NULL;
-	g_async_queue_push (self->priv->buffers, (_tmp0_ = buffer, (_tmp0_ == NULL) ? NULL : gst_buffer_ref (_tmp0_)));
+	g_async_queue_push (self->priv->buffers, _gst_buffer_ref0 (buffer));
 	g_idle_add_full (G_PRIORITY_HIGH_IDLE, _rygel_live_response_idle_handler_gsource_func, g_object_ref (self), g_object_unref);
 }
 
 
 static gboolean rygel_live_response_idle_handler (RygelLiveResponse* self) {
+	gboolean result;
 	GstBuffer* buffer;
-	gboolean _tmp0_;
 	g_return_val_if_fail (self != NULL, FALSE);
 	buffer = (GstBuffer*) g_async_queue_try_pop (self->priv->buffers);
 	if (buffer != NULL) {
 		rygel_http_response_push_data ((RygelHTTPResponse*) self, buffer->data, (gsize) buffer->size);
 	}
-	return (_tmp0_ = FALSE, (buffer == NULL) ? NULL : (buffer = (gst_buffer_unref (buffer), NULL)), _tmp0_);
+	result = FALSE;
+	_gst_buffer_unref0 (buffer);
+	return result;
 }
 
 
 static gboolean rygel_live_response_bus_handler (RygelLiveResponse* self, GstBus* bus, GstMessage* message) {
+	gboolean result;
 	gboolean ret;
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (bus != NULL, FALSE);
@@ -507,45 +545,92 @@ static gboolean rygel_live_response_bus_handler (RygelLiveResponse* self, GstBus
 	if (message->type == GST_MESSAGE_EOS) {
 		ret = FALSE;
 	} else {
-		GError* err;
-		char* err_msg;
-		err = NULL;
-		err_msg = NULL;
-		if (message->type == GST_MESSAGE_ERROR) {
-			char* _tmp3_;
-			char* _tmp2_;
-			GError* _tmp1_;
-			GError* _tmp0_;
-			_tmp3_ = NULL;
-			_tmp2_ = NULL;
-			_tmp1_ = NULL;
-			_tmp0_ = NULL;
-			(gst_message_parse_error (message, &_tmp0_, &_tmp2_), err = (_tmp1_ = _tmp0_, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), _tmp1_));
-			err_msg = (_tmp3_ = _tmp2_, err_msg = (g_free (err_msg), NULL), _tmp3_);
-			g_critical ("rygel-live-response.vala:235: Error from pipeline %s:%s", gst_object_get_name ((GstObject*) self->priv->pipeline), err_msg);
-			ret = FALSE;
-		} else {
-			if (message->type == GST_MESSAGE_WARNING) {
-				char* _tmp7_;
-				char* _tmp6_;
-				GError* _tmp5_;
-				GError* _tmp4_;
-				_tmp7_ = NULL;
-				_tmp6_ = NULL;
-				_tmp5_ = NULL;
-				_tmp4_ = NULL;
-				(gst_message_parse_warning (message, &_tmp4_, &_tmp6_), err = (_tmp5_ = _tmp4_, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), _tmp5_));
-				err_msg = (_tmp7_ = _tmp6_, err_msg = (g_free (err_msg), NULL), _tmp7_);
-				g_warning ("rygel-live-response.vala:242: Warning from pipeline %s:%s", gst_object_get_name ((GstObject*) self->priv->pipeline), err_msg);
+		if (message->type == GST_MESSAGE_STATE_CHANGED) {
+			gboolean _tmp0_;
+			_tmp0_ = FALSE;
+			if (self->priv->time_range != NULL) {
+				_tmp0_ = rygel_http_seek_get_start (self->priv->time_range) > 0;
+			} else {
+				_tmp0_ = FALSE;
 			}
+			if (_tmp0_) {
+				GstState old_state;
+				GstState new_state;
+				gboolean _tmp1_;
+				old_state = 0;
+				new_state = 0;
+				gst_message_parse_state_changed (message, &old_state, &new_state, NULL);
+				_tmp1_ = FALSE;
+				if (old_state == GST_STATE_READY) {
+					_tmp1_ = new_state == GST_STATE_PAUSED;
+				} else {
+					_tmp1_ = FALSE;
+				}
+				if (_tmp1_) {
+					if (rygel_live_response_seek (self)) {
+						gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_PLAYING);
+					}
+				}
+			}
+		} else {
+			GError* err;
+			char* err_msg;
+			err = NULL;
+			err_msg = NULL;
+			if (message->type == GST_MESSAGE_ERROR) {
+				char* _tmp5_;
+				char* _tmp4_;
+				GError* _tmp3_;
+				GError* _tmp2_;
+				_tmp4_ = NULL;
+				_tmp2_ = NULL;
+				(gst_message_parse_error (message, &_tmp2_, &_tmp4_), err = (_tmp3_ = _tmp2_, _g_error_free0 (err), _tmp3_));
+				err_msg = (_tmp5_ = _tmp4_, _g_free0 (err_msg), _tmp5_);
+				g_critical ("rygel-live-response.vala:258: Error from pipeline %s:%s", gst_object_get_name ((GstObject*) self->priv->pipeline), err_msg);
+				ret = FALSE;
+			} else {
+				if (message->type == GST_MESSAGE_WARNING) {
+					char* _tmp9_;
+					char* _tmp8_;
+					GError* _tmp7_;
+					GError* _tmp6_;
+					_tmp8_ = NULL;
+					_tmp6_ = NULL;
+					(gst_message_parse_warning (message, &_tmp6_, &_tmp8_), err = (_tmp7_ = _tmp6_, _g_error_free0 (err), _tmp7_));
+					err_msg = (_tmp9_ = _tmp8_, _g_free0 (err_msg), _tmp9_);
+					g_warning ("rygel-live-response.vala:265: Warning from pipeline %s:%s", gst_object_get_name ((GstObject*) self->priv->pipeline), err_msg);
+				}
+			}
+			_g_error_free0 (err);
+			_g_free0 (err_msg);
 		}
-		(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
-		err_msg = (g_free (err_msg), NULL);
 	}
 	if (!ret) {
 		rygel_http_response_end ((RygelHTTPResponse*) self, FALSE, (guint) SOUP_STATUS_NONE);
 	}
-	return ret;
+	result = ret;
+	return result;
+}
+
+
+static gboolean rygel_live_response_seek (RygelLiveResponse* self) {
+	gboolean result;
+	GstSeekType stop_type;
+	g_return_val_if_fail (self != NULL, FALSE);
+	stop_type = 0;
+	if (rygel_http_seek_get_stop (self->priv->time_range) > 0) {
+		stop_type = GST_SEEK_TYPE_SET;
+	} else {
+		stop_type = GST_SEEK_TYPE_NONE;
+	}
+	if (!gst_element_seek ((GstElement*) self->priv->pipeline, 1.0, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, rygel_http_seek_get_start (self->priv->time_range), stop_type, rygel_http_seek_get_stop (self->priv->time_range))) {
+		g_warning ("rygel-live-response.vala:294: Failed to seek to offset %lld", rygel_http_seek_get_start (self->priv->time_range));
+		rygel_http_response_end ((RygelHTTPResponse*) self, FALSE, (guint) SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE);
+		result = FALSE;
+		return result;
+	}
+	result = TRUE;
+	return result;
 }
 
 
@@ -566,8 +651,9 @@ static void rygel_live_response_instance_init (RygelLiveResponse * self) {
 static void rygel_live_response_finalize (GObject* obj) {
 	RygelLiveResponse * self;
 	self = RYGEL_LIVE_RESPONSE (obj);
-	(self->priv->pipeline == NULL) ? NULL : (self->priv->pipeline = (gst_object_unref (self->priv->pipeline), NULL));
-	(self->priv->buffers == NULL) ? NULL : (self->priv->buffers = (g_async_queue_unref (self->priv->buffers), NULL));
+	_gst_object_unref0 (self->priv->pipeline);
+	_g_async_queue_unref0 (self->priv->buffers);
+	_g_object_unref0 (self->priv->time_range);
 	G_OBJECT_CLASS (rygel_live_response_parent_class)->finalize (obj);
 }
 
