@@ -20,21 +20,89 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include "rygel-media-container.h"
+#include <glib.h>
+#include <glib-object.h>
+#include <stdlib.h>
+#include <string.h>
+#include <gee.h>
+#include <gio/gio.h>
 
 
+#define RYGEL_TYPE_MEDIA_OBJECT (rygel_media_object_get_type ())
+#define RYGEL_MEDIA_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_MEDIA_OBJECT, RygelMediaObject))
+#define RYGEL_MEDIA_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_MEDIA_OBJECT, RygelMediaObjectClass))
+#define RYGEL_IS_MEDIA_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_MEDIA_OBJECT))
+#define RYGEL_IS_MEDIA_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_MEDIA_OBJECT))
+#define RYGEL_MEDIA_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_MEDIA_OBJECT, RygelMediaObjectClass))
+
+typedef struct _RygelMediaObject RygelMediaObject;
+typedef struct _RygelMediaObjectClass RygelMediaObjectClass;
+typedef struct _RygelMediaObjectPrivate RygelMediaObjectPrivate;
+
+#define RYGEL_TYPE_MEDIA_CONTAINER (rygel_media_container_get_type ())
+#define RYGEL_MEDIA_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_MEDIA_CONTAINER, RygelMediaContainer))
+#define RYGEL_MEDIA_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_MEDIA_CONTAINER, RygelMediaContainerClass))
+#define RYGEL_IS_MEDIA_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_MEDIA_CONTAINER))
+#define RYGEL_IS_MEDIA_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_MEDIA_CONTAINER))
+#define RYGEL_MEDIA_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_MEDIA_CONTAINER, RygelMediaContainerClass))
+
+typedef struct _RygelMediaContainer RygelMediaContainer;
+typedef struct _RygelMediaContainerClass RygelMediaContainerClass;
+typedef struct _RygelMediaContainerPrivate RygelMediaContainerPrivate;
+#define _g_free0(var) (var = (g_free (var), NULL))
+
+struct _RygelMediaObject {
+	GObject parent_instance;
+	RygelMediaObjectPrivate * priv;
+	char* id;
+	char* title;
+	guint64 modified;
+	GeeArrayList* uris;
+	RygelMediaContainer* parent;
+	RygelMediaContainer* parent_ref;
+};
+
+struct _RygelMediaObjectClass {
+	GObjectClass parent_class;
+};
+
+struct _RygelMediaContainer {
+	RygelMediaObject parent_instance;
+	RygelMediaContainerPrivate * priv;
+	guint child_count;
+	guint32 update_id;
+};
+
+struct _RygelMediaContainerClass {
+	RygelMediaObjectClass parent_class;
+	void (*get_children) (RygelMediaContainer* self, guint offset, guint max_count, GCancellable* cancellable, GAsyncReadyCallback callback, void* callback_target);
+	GeeList* (*get_children_finish) (RygelMediaContainer* self, GAsyncResult* res, GError** error);
+	void (*find_object) (RygelMediaContainer* self, const char* id, GCancellable* cancellable, GAsyncReadyCallback callback, void* callback_target);
+	RygelMediaObject* (*find_object_finish) (RygelMediaContainer* self, GAsyncResult* res, GError** error);
+};
 
 
+static gpointer rygel_media_container_parent_class = NULL;
+
+GType rygel_media_object_get_type (void);
+GType rygel_media_container_get_type (void);
 enum  {
 	RYGEL_MEDIA_CONTAINER_DUMMY_PROPERTY
 };
-static void _rygel_media_container_on_container_updated_rygel_media_container_container_updated (RygelMediaContainer* _sender, RygelMediaContainer* container, gpointer self);
-static void rygel_media_container_real_get_children (RygelMediaContainer* self, guint offset, guint max_count, GCancellable* cancellable, GAsyncReadyCallback callback, void* callback_target);
-static GeeList* rygel_media_container_real_get_children_finish (RygelMediaContainer* self, GAsyncResult* res, GError** error);
-static void rygel_media_container_real_find_object (RygelMediaContainer* self, const char* id, GCancellable* cancellable, GAsyncReadyCallback callback, void* callback_target);
-static RygelMediaObject* rygel_media_container_real_find_object_finish (RygelMediaContainer* self, GAsyncResult* res, GError** error);
+RygelMediaObject* rygel_media_object_construct (GType object_type);
 static void rygel_media_container_on_container_updated (RygelMediaContainer* self, RygelMediaContainer* container, RygelMediaContainer* updated_container);
-static gpointer rygel_media_container_parent_class = NULL;
+static void _rygel_media_container_on_container_updated_rygel_media_container_container_updated (RygelMediaContainer* _sender, RygelMediaContainer* container, gpointer self);
+RygelMediaContainer* rygel_media_container_construct (GType object_type, const char* id, RygelMediaContainer* parent, const char* title, guint child_count);
+RygelMediaContainer* rygel_media_container_construct_root (GType object_type, const char* title, guint child_count);
+void rygel_media_container_get_children (RygelMediaContainer* self, guint offset, guint max_count, GCancellable* cancellable, GAsyncReadyCallback callback, void* callback_target);
+static void rygel_media_container_real_get_children (RygelMediaContainer* self, guint offset, guint max_count, GCancellable* cancellable, GAsyncReadyCallback callback, void* callback_target);
+GeeList* rygel_media_container_get_children_finish (RygelMediaContainer* self, GAsyncResult* res, GError** error);
+static GeeList* rygel_media_container_real_get_children_finish (RygelMediaContainer* self, GAsyncResult* res, GError** error);
+void rygel_media_container_find_object (RygelMediaContainer* self, const char* id, GCancellable* cancellable, GAsyncReadyCallback callback, void* callback_target);
+static void rygel_media_container_real_find_object (RygelMediaContainer* self, const char* id, GCancellable* cancellable, GAsyncReadyCallback callback, void* callback_target);
+RygelMediaObject* rygel_media_container_find_object_finish (RygelMediaContainer* self, GAsyncResult* res, GError** error);
+static RygelMediaObject* rygel_media_container_real_find_object_finish (RygelMediaContainer* self, GAsyncResult* res, GError** error);
+void rygel_media_container_updated (RygelMediaContainer* self);
 static void rygel_media_container_finalize (GObject* obj);
 
 
@@ -46,20 +114,14 @@ static void _rygel_media_container_on_container_updated_rygel_media_container_co
 
 RygelMediaContainer* rygel_media_container_construct (GType object_type, const char* id, RygelMediaContainer* parent, const char* title, guint child_count) {
 	RygelMediaContainer * self;
-	char* _tmp1;
-	const char* _tmp0;
-	char* _tmp3;
-	const char* _tmp2;
+	char* _tmp0_;
+	char* _tmp1_;
 	g_return_val_if_fail (id != NULL, NULL);
 	g_return_val_if_fail (title != NULL, NULL);
-	self = g_object_newv (object_type, 0, NULL);
-	_tmp1 = NULL;
-	_tmp0 = NULL;
-	((RygelMediaObject*) self)->id = (_tmp1 = (_tmp0 = id, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), ((RygelMediaObject*) self)->id = (g_free (((RygelMediaObject*) self)->id), NULL), _tmp1);
+	self = (RygelMediaContainer*) rygel_media_object_construct (object_type);
+	((RygelMediaObject*) self)->id = (_tmp0_ = g_strdup (id), _g_free0 (((RygelMediaObject*) self)->id), _tmp0_);
 	((RygelMediaObject*) self)->parent = parent;
-	_tmp3 = NULL;
-	_tmp2 = NULL;
-	((RygelMediaObject*) self)->title = (_tmp3 = (_tmp2 = title, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), ((RygelMediaObject*) self)->title = (g_free (((RygelMediaObject*) self)->title), NULL), _tmp3);
+	((RygelMediaObject*) self)->title = (_tmp1_ = g_strdup (title), _g_free0 (((RygelMediaObject*) self)->title), _tmp1_);
 	self->child_count = child_count;
 	self->update_id = (guint32) 0;
 	g_signal_connect_object (self, "container-updated", (GCallback) _rygel_media_container_on_container_updated_rygel_media_container_container_updated, self, 0);
@@ -67,11 +129,6 @@ RygelMediaContainer* rygel_media_container_construct (GType object_type, const c
 }
 
 
-RygelMediaContainer* rygel_media_container_new (const char* id, RygelMediaContainer* parent, const char* title, guint child_count) {
-	return rygel_media_container_construct (RYGEL_TYPE_MEDIA_CONTAINER, id, parent, title, child_count);
-}
-
-
 RygelMediaContainer* rygel_media_container_construct_root (GType object_type, const char* title, guint child_count) {
 	RygelMediaContainer * self;
 	g_return_val_if_fail (title != NULL, NULL);
@@ -80,11 +137,6 @@ RygelMediaContainer* rygel_media_container_construct_root (GType object_type, co
 }
 
 
-RygelMediaContainer* rygel_media_container_new_root (const char* title, guint child_count) {
-	return rygel_media_container_construct_root (RYGEL_TYPE_MEDIA_CONTAINER, title, child_count);
-}
-
-
 static void rygel_media_container_real_get_children (RygelMediaContainer* self, guint offset, guint max_count, GCancellable* cancellable, GAsyncReadyCallback callback, void* callback_target) {
 	g_return_if_fail (self != NULL);
 	g_critical ("Type `%s' does not implement abstract method `rygel_media_container_get_children'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
@@ -92,15 +144,6 @@ static void rygel_media_container_real_get_children (RygelMediaContainer* self,
 }
 
 
-/**
-     * Fetches the list of media objects directly under this container and
-     * calls callback once the result is ready.
-     *
-     * @param offet zero-based index of the first item to return
-     * @param max_count maximum number of objects to return
-     * @param cancellable optional cancellable for this operation
-     * @param callback function to call when result is ready
-     */
 void rygel_media_container_get_children (RygelMediaContainer* self, guint offset, guint max_count, GCancellable* cancellable, GAsyncReadyCallback callback, void* callback_target) {
 	RYGEL_MEDIA_CONTAINER_GET_CLASS (self)->get_children (self, offset, max_count, cancellable, callback, callback_target);
 }
@@ -113,13 +156,6 @@ static GeeList* rygel_media_container_real_get_children_finish (RygelMediaContai
 }
 
 
-/**
-     * Finishes the operation started by #get_children.
-     *
-     * @param res an AsyncResult
-     *
-     * return A list of media objects.
-     */
 GeeList* rygel_media_container_get_children_finish (RygelMediaContainer* self, GAsyncResult* res, GError** error) {
 	return RYGEL_MEDIA_CONTAINER_GET_CLASS (self)->get_children_finish (self, res, error);
 }
@@ -132,15 +168,6 @@ static void rygel_media_container_real_find_object (RygelMediaContainer* self, c
 }
 
 
-/**
-    * Recursively searches for media object with the given id in this
-    * container and calls callback when the result is available.
-    *
-    * @param id ID of the media object to search for
-    * @param cancellable optional cancellable for this operation
-    * @param callback function to call when result is ready
-    *
-    */
 void rygel_media_container_find_object (RygelMediaContainer* self, const char* id, GCancellable* cancellable, GAsyncReadyCallback callback, void* callback_target) {
 	RYGEL_MEDIA_CONTAINER_GET_CLASS (self)->find_object (self, id, cancellable, callback, callback_target);
 }
@@ -153,41 +180,18 @@ static RygelMediaObject* rygel_media_container_real_find_object_finish (RygelMed
 }
 
 
-/**
-     * Finishes the search operation started by #find_object.
-     *
-     * @param res an AsyncResult
-     *
-     * return the found media object.
-     */
 RygelMediaObject* rygel_media_container_find_object_finish (RygelMediaContainer* self, GAsyncResult* res, GError** error) {
 	return RYGEL_MEDIA_CONTAINER_GET_CLASS (self)->find_object_finish (self, res, error);
 }
 
 
-/**
-     * Method to be be called each time this container is updated (metadata
-     * changes for this container, items under it gets removed/added or their
-     * metadata changes etc).
-     *
-     * @param container the container that just got updated.
-     */
 void rygel_media_container_updated (RygelMediaContainer* self) {
 	g_return_if_fail (self != NULL);
 	self->update_id++;
-	/* Emit the signal that will start the bump-up process for this event.*/
 	g_signal_emit_by_name (self, "container-updated", self);
 }
 
 
-/**
-     * handler for container_updated signal on this container. We only forward
-     * it to the parent, hoping someone will get it from the root container
-     * and act upon it.
-     *
-     * @param container the container that emitted the signal
-     * @param updated_container the container that just got updated
-     */
 static void rygel_media_container_on_container_updated (RygelMediaContainer* self, RygelMediaContainer* container, RygelMediaContainer* updated_container) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (container != NULL);
@@ -200,11 +204,11 @@ static void rygel_media_container_on_container_updated (RygelMediaContainer* sel
 
 static void rygel_media_container_class_init (RygelMediaContainerClass * klass) {
 	rygel_media_container_parent_class = g_type_class_peek_parent (klass);
-	G_OBJECT_CLASS (klass)->finalize = rygel_media_container_finalize;
 	RYGEL_MEDIA_CONTAINER_CLASS (klass)->get_children = rygel_media_container_real_get_children;
 	RYGEL_MEDIA_CONTAINER_CLASS (klass)->get_children_finish = rygel_media_container_real_get_children_finish;
 	RYGEL_MEDIA_CONTAINER_CLASS (klass)->find_object = rygel_media_container_real_find_object;
 	RYGEL_MEDIA_CONTAINER_CLASS (klass)->find_object_finish = rygel_media_container_real_find_object_finish;
+	G_OBJECT_CLASS (klass)->finalize = rygel_media_container_finalize;
 	g_signal_new ("container_updated", RYGEL_TYPE_MEDIA_CONTAINER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, RYGEL_TYPE_MEDIA_CONTAINER);
 }
 
