@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Nokia Corporation, all rights reserved.
+ * Copyright (C) 2009 Nokia Corporation.
  *
  * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *                               <zeeshan.ali@nokia.com>
@@ -27,6 +27,7 @@
 #include <libgupnp-av/gupnp-av.h>
 #include <stdlib.h>
 #include <string.h>
+#include <gee.h>
 
 
 #define RYGEL_TYPE_TRANSCODER (rygel_transcoder_get_type ())
@@ -92,6 +93,39 @@ typedef struct _RygelMP3TranscoderPrivate RygelMP3TranscoderPrivate;
 
 typedef struct _RygelMP3TranscoderBin RygelMP3TranscoderBin;
 typedef struct _RygelMP3TranscoderBinClass RygelMP3TranscoderBinClass;
+#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
+typedef struct _RygelMediaObjectPrivate RygelMediaObjectPrivate;
+
+#define RYGEL_TYPE_MEDIA_CONTAINER (rygel_media_container_get_type ())
+#define RYGEL_MEDIA_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_MEDIA_CONTAINER, RygelMediaContainer))
+#define RYGEL_MEDIA_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_MEDIA_CONTAINER, RygelMediaContainerClass))
+#define RYGEL_IS_MEDIA_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_MEDIA_CONTAINER))
+#define RYGEL_IS_MEDIA_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_MEDIA_CONTAINER))
+#define RYGEL_MEDIA_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_MEDIA_CONTAINER, RygelMediaContainerClass))
+
+typedef struct _RygelMediaContainer RygelMediaContainer;
+typedef struct _RygelMediaContainerClass RygelMediaContainerClass;
+typedef struct _RygelMediaItemPrivate RygelMediaItemPrivate;
+
+#define RYGEL_TYPE_ICON_INFO (rygel_icon_info_get_type ())
+#define RYGEL_ICON_INFO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_ICON_INFO, RygelIconInfo))
+#define RYGEL_ICON_INFO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_ICON_INFO, RygelIconInfoClass))
+#define RYGEL_IS_ICON_INFO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_ICON_INFO))
+#define RYGEL_IS_ICON_INFO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_ICON_INFO))
+#define RYGEL_ICON_INFO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_ICON_INFO, RygelIconInfoClass))
+
+typedef struct _RygelIconInfo RygelIconInfo;
+typedef struct _RygelIconInfoClass RygelIconInfoClass;
+
+#define RYGEL_TYPE_THUMBNAIL (rygel_thumbnail_get_type ())
+#define RYGEL_THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_THUMBNAIL, RygelThumbnail))
+#define RYGEL_THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_THUMBNAIL, RygelThumbnailClass))
+#define RYGEL_IS_THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_THUMBNAIL))
+#define RYGEL_IS_THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_THUMBNAIL))
+#define RYGEL_THUMBNAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_THUMBNAIL, RygelThumbnailClass))
+
+typedef struct _RygelThumbnail RygelThumbnail;
+typedef struct _RygelThumbnailClass RygelThumbnailClass;
 
 #define TYPE_ENDIANNESS (endianness_get_type ())
 
@@ -104,11 +138,8 @@ typedef struct _RygelMP3TranscoderBinClass RygelMP3TranscoderBinClass;
 
 typedef struct _RygelL16Transcoder RygelL16Transcoder;
 typedef struct _RygelL16TranscoderClass RygelL16TranscoderClass;
+#define _gst_object_unref0(var) ((var == NULL) ? NULL : (var = (gst_object_unref (var), NULL)))
 
-/**
- * The base Transcoder class. Each implementation derives from it and must
- * at least implement create_source method.
- */
 struct _RygelTranscoder {
 	GObject parent_instance;
 	RygelTranscoderPrivate * priv;
@@ -116,14 +147,11 @@ struct _RygelTranscoder {
 
 struct _RygelTranscoderClass {
 	GObjectClass parent_class;
-	GstElement* (*create_source) (RygelTranscoder* self, GstElement* src, GError** error);
-	GUPnPDIDLLiteResource (*create_resource) (RygelTranscoder* self, RygelMediaItem* item, RygelTranscodeManager* manager, GError** error);
+	GstElement* (*create_source) (RygelTranscoder* self, RygelMediaItem* item, GstElement* src, GError** error);
+	GUPnPDIDLLiteResource* (*add_resource) (RygelTranscoder* self, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, RygelTranscodeManager* manager, GError** error);
+	guint (*get_distance) (RygelTranscoder* self, RygelMediaItem* item);
 };
 
-/**
- * Transcoder for mpeg 1 layer 2 and 3 audio. This element uses MP3TrancoderBin
- * for actual transcoding.
- */
 struct _RygelMP3Transcoder {
 	RygelTranscoder parent_instance;
 	RygelMP3TranscoderPrivate * priv;
@@ -142,12 +170,58 @@ struct _RygelMP3TranscoderPrivate {
 	RygelMP3Layer layer;
 };
 
+struct _RygelMediaObject {
+	GObject parent_instance;
+	RygelMediaObjectPrivate * priv;
+	char* id;
+	char* title;
+	guint64 modified;
+	GeeArrayList* uris;
+	RygelMediaContainer* parent;
+	RygelMediaContainer* parent_ref;
+};
+
+struct _RygelMediaObjectClass {
+	GObjectClass parent_class;
+};
+
+struct _RygelMediaItem {
+	RygelMediaObject parent_instance;
+	RygelMediaItemPrivate * priv;
+	char* author;
+	char* album;
+	char* date;
+	char* upnp_class;
+	char* mime_type;
+	char* dlna_profile;
+	glong size;
+	glong duration;
+	gint bitrate;
+	gint sample_freq;
+	gint bits_per_sample;
+	gint n_audio_channels;
+	gint track_number;
+	gint width;
+	gint height;
+	gint pixel_width;
+	gint pixel_height;
+	gint color_depth;
+	GeeArrayList* thumbnails;
+};
+
+struct _RygelMediaItemClass {
+	RygelMediaObjectClass parent_class;
+	GstElement* (*create_stream_source) (RygelMediaItem* self);
+	gboolean (*should_stream) (RygelMediaItem* self);
+};
+
 typedef enum  {
 	ENDIANNESS_LITTLE = G_LITTLE_ENDIAN,
 	ENDIANNESS_BIG = G_BIG_ENDIAN
 } Endianness;
 
 
+static gpointer rygel_mp3_transcoder_parent_class = NULL;
 
 GType rygel_transcoder_get_type (void);
 GType rygel_media_object_get_type (void);
@@ -163,27 +237,34 @@ enum  {
 #define RYGEL_MP3_TRANSCODER_AUDIO_PARSER "mp3parse"
 #define RYGEL_MP3_TRANSCODER_CONVERT_SINK_PAD "convert-sink-pad"
 #define RYGEL_MEDIA_ITEM_AUDIO_CLASS "object.item.audioItem"
-RygelTranscoder* rygel_transcoder_new (const char* mime_type, const char* dlna_profile, const char* upnp_class);
 RygelTranscoder* rygel_transcoder_construct (GType object_type, const char* mime_type, const char* dlna_profile, const char* upnp_class);
 RygelMP3Transcoder* rygel_mp3_transcoder_new (RygelMP3Layer layer);
 RygelMP3Transcoder* rygel_mp3_transcoder_construct (GType object_type, RygelMP3Layer layer);
-RygelMP3Transcoder* rygel_mp3_transcoder_new (RygelMP3Layer layer);
-RygelMP3TranscoderBin* rygel_mp3_transcoder_bin_new (GstElement* src, RygelMP3Transcoder* transcoder, GError** error);
-RygelMP3TranscoderBin* rygel_mp3_transcoder_bin_construct (GType object_type, GstElement* src, RygelMP3Transcoder* transcoder, GError** error);
+RygelMP3TranscoderBin* rygel_mp3_transcoder_bin_new (RygelMediaItem* item, GstElement* src, RygelMP3Transcoder* transcoder, GError** error);
+RygelMP3TranscoderBin* rygel_mp3_transcoder_bin_construct (GType object_type, RygelMediaItem* item, GstElement* src, RygelMP3Transcoder* transcoder, GError** error);
 GType rygel_mp3_transcoder_bin_get_type (void);
-static GstElement* rygel_mp3_transcoder_real_create_source (RygelTranscoder* base, GstElement* src, GError** error);
-GUPnPDIDLLiteResource rygel_transcoder_create_resource (RygelTranscoder* self, RygelMediaItem* item, RygelTranscodeManager* manager, GError** error);
-static GUPnPDIDLLiteResource rygel_mp3_transcoder_real_create_resource (RygelTranscoder* base, RygelMediaItem* item, RygelTranscodeManager* manager, GError** error);
+static GstElement* rygel_mp3_transcoder_real_create_source (RygelTranscoder* base, RygelMediaItem* item, GstElement* src, GError** error);
+GUPnPDIDLLiteResource* rygel_transcoder_add_resource (RygelTranscoder* self, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, RygelTranscodeManager* manager, GError** error);
+static GUPnPDIDLLiteResource* rygel_mp3_transcoder_real_add_resource (RygelTranscoder* base, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, RygelTranscodeManager* manager, GError** error);
+GType rygel_media_container_get_type (void);
+gpointer rygel_icon_info_ref (gpointer instance);
+void rygel_icon_info_unref (gpointer instance);
+GParamSpec* rygel_param_spec_icon_info (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void rygel_value_set_icon_info (GValue* value, gpointer v_object);
+gpointer rygel_value_get_icon_info (const GValue* value);
+GType rygel_icon_info_get_type (void);
+GType rygel_thumbnail_get_type (void);
+#define RYGEL_MEDIA_ITEM_IMAGE_CLASS "object.item.imageItem"
+static guint rygel_mp3_transcoder_real_get_distance (RygelTranscoder* base, RygelMediaItem* item);
 GType endianness_get_type (void);
 RygelL16Transcoder* rygel_l16_transcoder_new (Endianness endianness);
 RygelL16Transcoder* rygel_l16_transcoder_construct (GType object_type, Endianness endianness);
 GType rygel_l16_transcoder_get_type (void);
-GstElement* rygel_l16_transcoder_create_encoder (RygelL16Transcoder* self, const char* src_pad_name, const char* sink_pad_name, GError** error);
+GstElement* rygel_l16_transcoder_create_encoder (RygelL16Transcoder* self, RygelMediaItem* item, const char* src_pad_name, const char* sink_pad_name, GError** error);
 GstElement* rygel_gst_utils_create_element (const char* factoryname, const char* name, GError** error);
-static inline void _dynamic_set_quality2 (GstElement* obj, gint value);
-static inline void _dynamic_set_bitrate3 (GstElement* obj, gint value);
-GstElement* rygel_mp3_transcoder_create_encoder (RygelMP3Transcoder* self, const char* src_pad_name, const char* sink_pad_name, GError** error);
-static gpointer rygel_mp3_transcoder_parent_class = NULL;
+static inline void _dynamic_set_quality3 (GstElement* obj, gint value);
+static inline void _dynamic_set_bitrate4 (GstElement* obj, gint value);
+GstElement* rygel_mp3_transcoder_create_encoder (RygelMP3Transcoder* self, RygelMediaItem* item, const char* src_pad_name, const char* sink_pad_name, GError** error);
 static void rygel_mp3_transcoder_finalize (GObject* obj);
 
 static const char* RYGEL_MP3_TRANSCODER_AUDIO_ENCODER[] = {NULL, "twolame", "lame"};
@@ -202,106 +283,148 @@ RygelMP3Transcoder* rygel_mp3_transcoder_new (RygelMP3Layer layer) {
 }
 
 
-static GstElement* rygel_mp3_transcoder_real_create_source (RygelTranscoder* base, GstElement* src, GError** error) {
+static GstElement* rygel_mp3_transcoder_real_create_source (RygelTranscoder* base, RygelMediaItem* item, GstElement* src, GError** error) {
 	RygelMP3Transcoder * self;
+	GstElement* result;
 	GError * _inner_error_;
+	RygelMP3TranscoderBin* _tmp0_;
 	self = (RygelMP3Transcoder*) base;
+	g_return_val_if_fail (item != NULL, NULL);
 	g_return_val_if_fail (src != NULL, NULL);
 	_inner_error_ = NULL;
-	return (GstElement*) rygel_mp3_transcoder_bin_new (src, self, &_inner_error_);
-}
-
-
-static GUPnPDIDLLiteResource rygel_mp3_transcoder_real_create_resource (RygelTranscoder* base, RygelMediaItem* item, RygelTranscodeManager* manager, GError** error) {
-	RygelMP3Transcoder * self;
-	GError * _inner_error_;
-	GUPnPDIDLLiteResource res;
-	self = (RygelMP3Transcoder*) base;
-	_inner_error_ = NULL;
-	res = RYGEL_TRANSCODER_CLASS (rygel_mp3_transcoder_parent_class)->create_resource (RYGEL_TRANSCODER (self), item, manager, &_inner_error_);
+	_tmp0_ = rygel_mp3_transcoder_bin_new (item, src, self, &_inner_error_);
 	if (_inner_error_ != NULL) {
 		g_propagate_error (error, _inner_error_);
-		return;
+		return NULL;
 	}
-	res.bitrate = RYGEL_MP3_TRANSCODER_BITRATE;
-	return res;
+	result = (GstElement*) _tmp0_;
+	return result;
 }
 
 
-static inline void _dynamic_set_quality2 (GstElement* obj, gint value) {
+static GUPnPDIDLLiteResource* rygel_mp3_transcoder_real_add_resource (RygelTranscoder* base, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, RygelTranscodeManager* manager, GError** error) {
+	RygelMP3Transcoder * self;
+	GUPnPDIDLLiteResource* result;
+	GError * _inner_error_;
+	GUPnPDIDLLiteResource* resource;
+	self = (RygelMP3Transcoder*) base;
+	g_return_val_if_fail (didl_item != NULL, NULL);
+	g_return_val_if_fail (item != NULL, NULL);
+	g_return_val_if_fail (manager != NULL, NULL);
+	_inner_error_ = NULL;
+	resource = RYGEL_TRANSCODER_CLASS (rygel_mp3_transcoder_parent_class)->add_resource (RYGEL_TRANSCODER (self), didl_item, item, manager, &_inner_error_);
+	if (_inner_error_ != NULL) {
+		g_propagate_error (error, _inner_error_);
+		return NULL;
+	}
+	if (resource == NULL) {
+		result = NULL;
+		_g_object_unref0 (resource);
+		return result;
+	}
+	gupnp_didl_lite_resource_set_bitrate (resource, (RYGEL_MP3_TRANSCODER_BITRATE * 1000) / 8);
+	result = resource;
+	return result;
+}
+
+
+static guint rygel_mp3_transcoder_real_get_distance (RygelTranscoder* base, RygelMediaItem* item) {
+	RygelMP3Transcoder * self;
+	guint result;
+	guint distance;
+	self = (RygelMP3Transcoder*) base;
+	g_return_val_if_fail (item != NULL, 0U);
+	if (g_str_has_prefix (item->upnp_class, RYGEL_MEDIA_ITEM_IMAGE_CLASS)) {
+		result = G_MAXUINT;
+		return result;
+	}
+	distance = 0U;
+	if (g_str_has_prefix (item->upnp_class, RYGEL_MEDIA_ITEM_AUDIO_CLASS)) {
+		distance = 0;
+		if (item->bitrate > 0) {
+			distance = distance + ((guint) abs (item->bitrate - RYGEL_MP3_TRANSCODER_BITRATE));
+		}
+	} else {
+		distance = G_MAXUINT / 2;
+	}
+	result = distance;
+	return result;
+}
+
+
+static inline void _dynamic_set_quality3 (GstElement* obj, gint value) {
 	g_object_set (obj, "quality", value, NULL);
 }
 
 
-static inline void _dynamic_set_bitrate3 (GstElement* obj, gint value) {
+static inline void _dynamic_set_bitrate4 (GstElement* obj, gint value) {
 	g_object_set (obj, "bitrate", value, NULL);
 }
 
 
-GstElement* rygel_mp3_transcoder_create_encoder (RygelMP3Transcoder* self, const char* src_pad_name, const char* sink_pad_name, GError** error) {
+static gpointer _gst_object_ref0 (gpointer self) {
+	return self ? gst_object_ref (self) : NULL;
+}
+
+
+GstElement* rygel_mp3_transcoder_create_encoder (RygelMP3Transcoder* self, RygelMediaItem* item, const char* src_pad_name, const char* sink_pad_name, GError** error) {
+	GstElement* result;
 	GError * _inner_error_;
 	RygelL16Transcoder* l16_transcoder;
 	GstElement* convert;
 	GstElement* encoder;
 	GstElement* parser;
 	GstBin* bin;
-	GstElement* _tmp2_;
-	GstElement* _tmp1_;
-	GstElement* _tmp0_;
 	GstPad* pad;
 	GstGhostPad* ghost;
-	GstPad* _tmp3_;
-	GstPad* _tmp4_;
-	GstGhostPad* _tmp5_;
-	GstPad* _tmp6_;
-	GstElement* _tmp7_;
+	GstPad* _tmp0_;
+	GstGhostPad* _tmp1_;
 	g_return_val_if_fail (self != NULL, NULL);
+	g_return_val_if_fail (item != NULL, NULL);
 	_inner_error_ = NULL;
 	l16_transcoder = rygel_l16_transcoder_new (ENDIANNESS_LITTLE);
-	convert = rygel_l16_transcoder_create_encoder (l16_transcoder, NULL, RYGEL_MP3_TRANSCODER_CONVERT_SINK_PAD, &_inner_error_);
+	convert = rygel_l16_transcoder_create_encoder (l16_transcoder, item, NULL, RYGEL_MP3_TRANSCODER_CONVERT_SINK_PAD, &_inner_error_);
 	if (_inner_error_ != NULL) {
 		g_propagate_error (error, _inner_error_);
-		(l16_transcoder == NULL) ? NULL : (l16_transcoder = (g_object_unref (l16_transcoder), NULL));
+		_g_object_unref0 (l16_transcoder);
 		return NULL;
 	}
 	encoder = rygel_gst_utils_create_element (RYGEL_MP3_TRANSCODER_AUDIO_ENCODER[self->priv->layer], RYGEL_MP3_TRANSCODER_AUDIO_ENCODER[self->priv->layer], &_inner_error_);
 	if (_inner_error_ != NULL) {
 		g_propagate_error (error, _inner_error_);
-		(l16_transcoder == NULL) ? NULL : (l16_transcoder = (g_object_unref (l16_transcoder), NULL));
-		(convert == NULL) ? NULL : (convert = (gst_object_unref (convert), NULL));
+		_g_object_unref0 (l16_transcoder);
+		_gst_object_unref0 (convert);
 		return NULL;
 	}
 	parser = rygel_gst_utils_create_element (RYGEL_MP3_TRANSCODER_AUDIO_PARSER, RYGEL_MP3_TRANSCODER_AUDIO_PARSER, &_inner_error_);
 	if (_inner_error_ != NULL) {
 		g_propagate_error (error, _inner_error_);
-		(l16_transcoder == NULL) ? NULL : (l16_transcoder = (g_object_unref (l16_transcoder), NULL));
-		(convert == NULL) ? NULL : (convert = (gst_object_unref (convert), NULL));
-		(encoder == NULL) ? NULL : (encoder = (gst_object_unref (encoder), NULL));
+		_g_object_unref0 (l16_transcoder);
+		_gst_object_unref0 (convert);
+		_gst_object_unref0 (encoder);
 		return NULL;
 	}
 	if (self->priv->layer == RYGEL_MP3_LAYER_THREE) {
-		/* Best quality*/
-		_dynamic_set_quality2 (encoder, 0);
+		_dynamic_set_quality3 (encoder, 0);
 	}
-	_dynamic_set_bitrate3 (encoder, RYGEL_MP3_TRANSCODER_BITRATE);
+	_dynamic_set_bitrate4 (encoder, RYGEL_MP3_TRANSCODER_BITRATE);
 	bin = (GstBin*) gst_bin_new ("mp3-encoder-bin");
-	_tmp2_ = NULL;
-	_tmp1_ = NULL;
-	_tmp0_ = NULL;
-	gst_bin_add_many (bin, (_tmp0_ = convert, (_tmp0_ == NULL) ? NULL : gst_object_ref (_tmp0_)), (_tmp1_ = encoder, (_tmp1_ == NULL) ? NULL : gst_object_ref (_tmp1_)), (_tmp2_ = parser, (_tmp2_ == NULL) ? NULL : gst_object_ref (_tmp2_)), NULL);
+	gst_bin_add_many (bin, _gst_object_ref0 (convert), _gst_object_ref0 (encoder), _gst_object_ref0 (parser), NULL);
 	gst_element_link_many (convert, encoder, parser, NULL);
 	pad = gst_element_get_static_pad (convert, RYGEL_MP3_TRANSCODER_CONVERT_SINK_PAD);
 	ghost = (GstGhostPad*) gst_ghost_pad_new (sink_pad_name, pad);
-	_tmp3_ = NULL;
-	gst_element_add_pad ((GstElement*) bin, (_tmp3_ = (GstPad*) ghost, (_tmp3_ == NULL) ? NULL : gst_object_ref (_tmp3_)));
-	_tmp4_ = NULL;
-	pad = (_tmp4_ = gst_element_get_static_pad (parser, "src"), (pad == NULL) ? NULL : (pad = (gst_object_unref (pad), NULL)), _tmp4_);
-	_tmp5_ = NULL;
-	ghost = (_tmp5_ = (GstGhostPad*) gst_ghost_pad_new (src_pad_name, pad), (ghost == NULL) ? NULL : (ghost = (gst_object_unref (ghost), NULL)), _tmp5_);
-	_tmp6_ = NULL;
-	gst_element_add_pad ((GstElement*) bin, (_tmp6_ = (GstPad*) ghost, (_tmp6_ == NULL) ? NULL : gst_object_ref (_tmp6_)));
-	_tmp7_ = NULL;
-	return (_tmp7_ = (GstElement*) bin, (l16_transcoder == NULL) ? NULL : (l16_transcoder = (g_object_unref (l16_transcoder), NULL)), (convert == NULL) ? NULL : (convert = (gst_object_unref (convert), NULL)), (encoder == NULL) ? NULL : (encoder = (gst_object_unref (encoder), NULL)), (parser == NULL) ? NULL : (parser = (gst_object_unref (parser), NULL)), (pad == NULL) ? NULL : (pad = (gst_object_unref (pad), NULL)), (ghost == NULL) ? NULL : (ghost = (gst_object_unref (ghost), NULL)), _tmp7_);
+	gst_element_add_pad ((GstElement*) bin, _gst_object_ref0 ((GstPad*) ghost));
+	pad = (_tmp0_ = gst_element_get_static_pad (parser, "src"), _gst_object_unref0 (pad), _tmp0_);
+	ghost = (_tmp1_ = (GstGhostPad*) gst_ghost_pad_new (src_pad_name, pad), _gst_object_unref0 (ghost), _tmp1_);
+	gst_element_add_pad ((GstElement*) bin, _gst_object_ref0 ((GstPad*) ghost));
+	result = (GstElement*) bin;
+	_g_object_unref0 (l16_transcoder);
+	_gst_object_unref0 (convert);
+	_gst_object_unref0 (encoder);
+	_gst_object_unref0 (parser);
+	_gst_object_unref0 (pad);
+	_gst_object_unref0 (ghost);
+	return result;
 }
 
 
@@ -309,7 +432,8 @@ static void rygel_mp3_transcoder_class_init (RygelMP3TranscoderClass * klass) {
 	rygel_mp3_transcoder_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (RygelMP3TranscoderPrivate));
 	RYGEL_TRANSCODER_CLASS (klass)->create_source = rygel_mp3_transcoder_real_create_source;
-	RYGEL_TRANSCODER_CLASS (klass)->create_resource = rygel_mp3_transcoder_real_create_resource;
+	RYGEL_TRANSCODER_CLASS (klass)->add_resource = rygel_mp3_transcoder_real_add_resource;
+	RYGEL_TRANSCODER_CLASS (klass)->get_distance = rygel_mp3_transcoder_real_get_distance;
 	G_OBJECT_CLASS (klass)->finalize = rygel_mp3_transcoder_finalize;
 }
 
