@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Nokia Corporation, all rights reserved.
+ * Copyright (C) 2009 Nokia Corporation.
  *
  * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *                               <zeeshan.ali@nokia.com>
@@ -25,12 +25,8 @@
 #include <glib-object.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gee/arraylist.h>
 #include <libgupnp-av/gupnp-av.h>
-#include <gee/collection.h>
-#include <gst/gst.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
+#include <gee.h>
 #include <libsoup/soup.h>
 
 
@@ -74,16 +70,25 @@ typedef struct _RygelMediaItemClass RygelMediaItemClass;
 
 typedef struct _RygelTranscoder RygelTranscoder;
 typedef struct _RygelTranscoderClass RygelTranscoderClass;
+#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
+
+#define RYGEL_TYPE_META_CONFIG (rygel_meta_config_get_type ())
+#define RYGEL_META_CONFIG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_META_CONFIG, RygelMetaConfig))
+#define RYGEL_META_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_META_CONFIG, RygelMetaConfigClass))
+#define RYGEL_IS_META_CONFIG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_META_CONFIG))
+#define RYGEL_IS_META_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_META_CONFIG))
+#define RYGEL_META_CONFIG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_META_CONFIG, RygelMetaConfigClass))
+
+typedef struct _RygelMetaConfig RygelMetaConfig;
+typedef struct _RygelMetaConfigClass RygelMetaConfigClass;
 
 #define RYGEL_TYPE_CONFIGURATION (rygel_configuration_get_type ())
 #define RYGEL_CONFIGURATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_CONFIGURATION, RygelConfiguration))
-#define RYGEL_CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_CONFIGURATION, RygelConfigurationClass))
 #define RYGEL_IS_CONFIGURATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_CONFIGURATION))
-#define RYGEL_IS_CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_CONFIGURATION))
-#define RYGEL_CONFIGURATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_CONFIGURATION, RygelConfigurationClass))
+#define RYGEL_CONFIGURATION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), RYGEL_TYPE_CONFIGURATION, RygelConfigurationIface))
 
 typedef struct _RygelConfiguration RygelConfiguration;
-typedef struct _RygelConfigurationClass RygelConfigurationClass;
+typedef struct _RygelConfigurationIface RygelConfigurationIface;
 
 #define TYPE_ENDIANNESS (endianness_get_type ())
 
@@ -120,24 +125,9 @@ typedef struct _RygelMP3TranscoderClass RygelMP3TranscoderClass;
 
 typedef struct _RygelMP2TSTranscoder RygelMP2TSTranscoder;
 typedef struct _RygelMP2TSTranscoderClass RygelMP2TSTranscoderClass;
-typedef struct _RygelMediaObjectPrivate RygelMediaObjectPrivate;
+#define __g_list_free_g_object_unref0(var) ((var == NULL) ? NULL : (var = (_g_list_free_g_object_unref (var), NULL)))
+#define _g_free0(var) (var = (g_free (var), NULL))
 
-#define RYGEL_TYPE_MEDIA_CONTAINER (rygel_media_container_get_type ())
-#define RYGEL_MEDIA_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_MEDIA_CONTAINER, RygelMediaContainer))
-#define RYGEL_MEDIA_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_MEDIA_CONTAINER, RygelMediaContainerClass))
-#define RYGEL_IS_MEDIA_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_MEDIA_CONTAINER))
-#define RYGEL_IS_MEDIA_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_MEDIA_CONTAINER))
-#define RYGEL_MEDIA_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_MEDIA_CONTAINER, RygelMediaContainerClass))
-
-typedef struct _RygelMediaContainer RygelMediaContainer;
-typedef struct _RygelMediaContainerClass RygelMediaContainerClass;
-typedef struct _RygelMediaItemPrivate RygelMediaItemPrivate;
-
-/**
- * Responsible for management of all transcoders:
- *    - gets the appropriate transcoder given a transcoding target.
- *    - provide all possible transcoding resources for items.
- */
 struct _RygelTranscodeManager {
 	GObject parent_instance;
 	RygelTranscodeManagerPrivate * priv;
@@ -145,14 +135,34 @@ struct _RygelTranscodeManager {
 
 struct _RygelTranscodeManagerClass {
 	GObjectClass parent_class;
-	char* (*create_uri_for_item) (RygelTranscodeManager* self, RygelMediaItem* item, const char* transcode_target, char** protocol);
-	void (*add_resources) (RygelTranscodeManager* self, GeeArrayList* resources, RygelMediaItem* item, GError** error);
+	char* (*create_uri_for_item) (RygelTranscodeManager* self, RygelMediaItem* item, gint thumbnail_index, const char* transcode_target, char** protocol);
+	void (*add_resources) (RygelTranscodeManager* self, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, GError** error);
+	char* (*get_protocol) (RygelTranscodeManager* self);
+	char* (*get_protocol_info) (RygelTranscodeManager* self);
 };
 
 struct _RygelTranscodeManagerPrivate {
 	GeeArrayList* transcoders;
 };
 
+struct _RygelConfigurationIface {
+	GTypeInterface parent_iface;
+	gboolean (*get_upnp_enabled) (RygelConfiguration* self, GError** error);
+	char* (*get_interface) (RygelConfiguration* self, GError** error);
+	gint (*get_port) (RygelConfiguration* self, GError** error);
+	gboolean (*get_transcoding) (RygelConfiguration* self, GError** error);
+	gboolean (*get_mp3_transcoder) (RygelConfiguration* self, GError** error);
+	gboolean (*get_mp2ts_transcoder) (RygelConfiguration* self, GError** error);
+	gboolean (*get_lpcm_transcoder) (RygelConfiguration* self, GError** error);
+	gboolean (*get_enabled) (RygelConfiguration* self, const char* section, GError** error);
+	char* (*get_title) (RygelConfiguration* self, const char* section, GError** error);
+	char* (*get_string) (RygelConfiguration* self, const char* section, const char* key, GError** error);
+	GeeArrayList* (*get_string_list) (RygelConfiguration* self, const char* section, const char* key, GError** error);
+	gint (*get_int) (RygelConfiguration* self, const char* section, const char* key, gint min, gint max, GError** error);
+	GeeArrayList* (*get_int_list) (RygelConfiguration* self, const char* section, const char* key, GError** error);
+	gboolean (*get_bool) (RygelConfiguration* self, const char* section, const char* key, GError** error);
+};
+
 typedef enum  {
 	ENDIANNESS_LITTLE = G_LITTLE_ENDIAN,
 	ENDIANNESS_BIG = G_BIG_ENDIAN
@@ -168,59 +178,14 @@ typedef enum  {
 	RYGEL_MP2_TS_PROFILE_HD
 } RygelMP2TSProfile;
 
-/**
- * Represents a media object (container and item).
- */
-struct _RygelMediaObject {
-	GObject parent_instance;
-	RygelMediaObjectPrivate * priv;
-	char* id;
-	char* title;
-	RygelMediaContainer* parent;
-};
-
-struct _RygelMediaObjectClass {
-	GObjectClass parent_class;
-};
-
-/**
- * Represents a media (Music, Video and Image) item.
- */
-struct _RygelMediaItem {
-	RygelMediaObject parent_instance;
-	RygelMediaItemPrivate * priv;
-	char* author;
-	char* album;
-	char* date;
-	char* upnp_class;
-	GeeArrayList* uris;
-	char* mime_type;
-	glong size;
-	glong duration;
-	gint bitrate;
-	gint sample_freq;
-	gint bits_per_sample;
-	gint n_audio_channels;
-	gint track_number;
-	gint width;
-	gint height;
-	gint color_depth;
-};
-
-struct _RygelMediaItemClass {
-	RygelMediaObjectClass parent_class;
-	GstElement* (*create_stream_source) (RygelMediaItem* self);
-};
-
 typedef enum  {
 	RYGEL_HTTP_REQUEST_ERROR_UNACCEPTABLE = SOUP_STATUS_NOT_ACCEPTABLE,
-	RYGEL_HTTP_REQUEST_ERROR_INVALID_RANGE = SOUP_STATUS_BAD_REQUEST,
-	RYGEL_HTTP_REQUEST_ERROR_OUT_OF_RANGE = SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE,
 	RYGEL_HTTP_REQUEST_ERROR_BAD_REQUEST = SOUP_STATUS_BAD_REQUEST,
 	RYGEL_HTTP_REQUEST_ERROR_NOT_FOUND = SOUP_STATUS_NOT_FOUND
 } RygelHTTPRequestError;
 #define RYGEL_HTTP_REQUEST_ERROR rygel_http_request_error_quark ()
 
+static gpointer rygel_transcode_manager_parent_class = NULL;
 
 GType rygel_transcode_manager_get_type (void);
 GType rygel_media_object_get_type (void);
@@ -230,166 +195,196 @@ GType rygel_transcoder_get_type (void);
 enum  {
 	RYGEL_TRANSCODE_MANAGER_DUMMY_PROPERTY
 };
+GType rygel_meta_config_get_type (void);
+RygelMetaConfig* rygel_meta_config_get_default (void);
 GType rygel_configuration_get_type (void);
-RygelConfiguration* rygel_configuration_get_default (void);
-gboolean rygel_configuration_get_transcoding (RygelConfiguration* self);
-gboolean rygel_configuration_get_lpcm_transcoder (RygelConfiguration* self);
+gboolean rygel_configuration_get_transcoding (RygelConfiguration* self, GError** error);
+gboolean rygel_configuration_get_lpcm_transcoder (RygelConfiguration* self, GError** error);
 GType endianness_get_type (void);
 RygelL16Transcoder* rygel_l16_transcoder_new (Endianness endianness);
 RygelL16Transcoder* rygel_l16_transcoder_construct (GType object_type, Endianness endianness);
 GType rygel_l16_transcoder_get_type (void);
-gboolean rygel_configuration_get_mp3_transcoder (RygelConfiguration* self);
+gboolean rygel_configuration_get_mp3_transcoder (RygelConfiguration* self, GError** error);
 GType rygel_mp3_layer_get_type (void);
 RygelMP3Transcoder* rygel_mp3_transcoder_new (RygelMP3Layer layer);
 RygelMP3Transcoder* rygel_mp3_transcoder_construct (GType object_type, RygelMP3Layer layer);
 GType rygel_mp3_transcoder_get_type (void);
-gboolean rygel_configuration_get_mp2ts_transcoder (RygelConfiguration* self);
+gboolean rygel_configuration_get_mp2ts_transcoder (RygelConfiguration* self, GError** error);
 GType rygel_mp2_ts_profile_get_type (void);
 RygelMP2TSTranscoder* rygel_mp2_ts_transcoder_new (RygelMP2TSProfile profile);
 RygelMP2TSTranscoder* rygel_mp2_ts_transcoder_construct (GType object_type, RygelMP2TSProfile profile);
 GType rygel_mp2_ts_transcoder_get_type (void);
-RygelTranscodeManager* rygel_transcode_manager_new (void);
-RygelTranscodeManager* rygel_transcode_manager_construct (GType object_type);
-RygelTranscodeManager* rygel_transcode_manager_new (void);
-char* rygel_transcode_manager_create_uri_for_item (RygelTranscodeManager* self, RygelMediaItem* item, const char* transcode_target, char** protocol);
-static char* rygel_transcode_manager_real_create_uri_for_item (RygelTranscodeManager* self, RygelMediaItem* item, const char* transcode_target, char** protocol);
-GType rygel_media_container_get_type (void);
-#define RYGEL_MEDIA_ITEM_IMAGE_CLASS "object.item.imageItem"
-const char* rygel_transcoder_get_upnp_class (RygelTranscoder* self);
-void rygel_transcoder_add_resources (RygelTranscoder* self, GeeArrayList* resources, RygelMediaItem* item, RygelTranscodeManager* manager, GError** error);
-void rygel_transcode_manager_add_resources (RygelTranscodeManager* self, GeeArrayList* resources, RygelMediaItem* item, GError** error);
-static void rygel_transcode_manager_real_add_resources (RygelTranscodeManager* self, GeeArrayList* resources, RygelMediaItem* item, GError** error);
+RygelTranscodeManager* rygel_transcode_manager_construct (GType object_type, GError** error);
+char* rygel_transcode_manager_create_uri_for_item (RygelTranscodeManager* self, RygelMediaItem* item, gint thumbnail_index, const char* transcode_target, char** protocol);
+static char* rygel_transcode_manager_real_create_uri_for_item (RygelTranscodeManager* self, RygelMediaItem* item, gint thumbnail_index, const char* transcode_target, char** protocol);
+guint rygel_transcoder_get_distance (RygelTranscoder* self, RygelMediaItem* item);
+gint rygel_media_item_compare_transcoders (RygelMediaItem* self, void* a, void* b);
+static gint _rygel_media_item_compare_transcoders_gcompare_data_func (void* a, void* b, gpointer self);
+GUPnPDIDLLiteResource* rygel_transcoder_add_resource (RygelTranscoder* self, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, RygelTranscodeManager* manager, GError** error);
+static void _g_list_free_g_object_unref (GList* self);
+void rygel_transcode_manager_add_resources (RygelTranscodeManager* self, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, GError** error);
+static void rygel_transcode_manager_real_add_resources (RygelTranscodeManager* self, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, GError** error);
 gboolean rygel_transcoder_can_handle (RygelTranscoder* self, const char* target);
 GQuark rygel_http_request_error_quark (void);
 RygelTranscoder* rygel_transcode_manager_get_transcoder (RygelTranscodeManager* self, const char* target, GError** error);
-static gpointer rygel_transcode_manager_parent_class = NULL;
+char* rygel_transcode_manager_get_protocol (RygelTranscodeManager* self);
+static char* rygel_transcode_manager_real_get_protocol (RygelTranscodeManager* self);
+const char* rygel_transcoder_get_mime_type (RygelTranscoder* self);
+const char* rygel_transcoder_get_dlna_profile (RygelTranscoder* self);
+char* rygel_transcode_manager_get_protocol_info (RygelTranscodeManager* self);
+static char* rygel_transcode_manager_real_get_protocol_info (RygelTranscodeManager* self);
 static void rygel_transcode_manager_finalize (GObject* obj);
+static int _vala_strcmp0 (const char * str1, const char * str2);
 
 
 
-RygelTranscodeManager* rygel_transcode_manager_construct (GType object_type) {
+RygelTranscodeManager* rygel_transcode_manager_construct (GType object_type, GError** error) {
+	GError * _inner_error_;
 	RygelTranscodeManager * self;
 	GeeArrayList* _tmp0_;
-	RygelConfiguration* config;
-	self = g_object_newv (object_type, 0, NULL);
-	_tmp0_ = NULL;
-	self->priv->transcoders = (_tmp0_ = gee_array_list_new (RYGEL_TYPE_TRANSCODER, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal), (self->priv->transcoders == NULL) ? NULL : (self->priv->transcoders = (g_object_unref (self->priv->transcoders), NULL)), _tmp0_);
-	config = rygel_configuration_get_default ();
-	if (rygel_configuration_get_transcoding (config)) {
-		if (rygel_configuration_get_lpcm_transcoder (config)) {
-			RygelL16Transcoder* _tmp1_;
-			_tmp1_ = NULL;
-			gee_collection_add ((GeeCollection*) self->priv->transcoders, (RygelTranscoder*) (_tmp1_ = rygel_l16_transcoder_new (ENDIANNESS_BIG)));
-			(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
+	RygelMetaConfig* config;
+	gboolean _tmp1_;
+	_inner_error_ = NULL;
+	self = (RygelTranscodeManager*) g_object_new (object_type, NULL);
+	self->priv->transcoders = (_tmp0_ = gee_array_list_new (RYGEL_TYPE_TRANSCODER, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal), _g_object_unref0 (self->priv->transcoders), _tmp0_);
+	config = rygel_meta_config_get_default ();
+	_tmp1_ = rygel_configuration_get_transcoding ((RygelConfiguration*) config, &_inner_error_);
+	if (_inner_error_ != NULL) {
+		g_propagate_error (error, _inner_error_);
+		_g_object_unref0 (config);
+		return;
+	}
+	if (_tmp1_) {
+		gboolean _tmp2_;
+		gboolean _tmp4_;
+		gboolean _tmp6_;
+		_tmp2_ = rygel_configuration_get_lpcm_transcoder ((RygelConfiguration*) config, &_inner_error_);
+		if (_inner_error_ != NULL) {
+			g_propagate_error (error, _inner_error_);
+			_g_object_unref0 (config);
+			return;
 		}
-		if (rygel_configuration_get_mp3_transcoder (config)) {
-			RygelMP3Transcoder* _tmp2_;
-			_tmp2_ = NULL;
-			gee_collection_add ((GeeCollection*) self->priv->transcoders, (RygelTranscoder*) (_tmp2_ = rygel_mp3_transcoder_new (RYGEL_MP3_LAYER_THREE)));
-			(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
+		if (_tmp2_) {
+			RygelL16Transcoder* _tmp3_;
+			gee_abstract_collection_add ((GeeAbstractCollection*) self->priv->transcoders, (RygelTranscoder*) (_tmp3_ = rygel_l16_transcoder_new (ENDIANNESS_BIG)));
+			_g_object_unref0 (_tmp3_);
 		}
-		if (rygel_configuration_get_mp2ts_transcoder (config)) {
-			RygelMP2TSTranscoder* _tmp3_;
-			RygelMP2TSTranscoder* _tmp4_;
-			_tmp3_ = NULL;
-			gee_collection_add ((GeeCollection*) self->priv->transcoders, (RygelTranscoder*) (_tmp3_ = rygel_mp2_ts_transcoder_new (RYGEL_MP2_TS_PROFILE_SD)));
-			(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
-			_tmp4_ = NULL;
-			gee_collection_add ((GeeCollection*) self->priv->transcoders, (RygelTranscoder*) (_tmp4_ = rygel_mp2_ts_transcoder_new (RYGEL_MP2_TS_PROFILE_HD)));
-			(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
+		_tmp4_ = rygel_configuration_get_mp3_transcoder ((RygelConfiguration*) config, &_inner_error_);
+		if (_inner_error_ != NULL) {
+			g_propagate_error (error, _inner_error_);
+			_g_object_unref0 (config);
+			return;
+		}
+		if (_tmp4_) {
+			RygelMP3Transcoder* _tmp5_;
+			gee_abstract_collection_add ((GeeAbstractCollection*) self->priv->transcoders, (RygelTranscoder*) (_tmp5_ = rygel_mp3_transcoder_new (RYGEL_MP3_LAYER_THREE)));
+			_g_object_unref0 (_tmp5_);
+		}
+		_tmp6_ = rygel_configuration_get_mp2ts_transcoder ((RygelConfiguration*) config, &_inner_error_);
+		if (_inner_error_ != NULL) {
+			g_propagate_error (error, _inner_error_);
+			_g_object_unref0 (config);
+			return;
+		}
+		if (_tmp6_) {
+			RygelMP2TSTranscoder* _tmp7_;
+			RygelMP2TSTranscoder* _tmp8_;
+			gee_abstract_collection_add ((GeeAbstractCollection*) self->priv->transcoders, (RygelTranscoder*) (_tmp7_ = rygel_mp2_ts_transcoder_new (RYGEL_MP2_TS_PROFILE_SD)));
+			_g_object_unref0 (_tmp7_);
+			gee_abstract_collection_add ((GeeAbstractCollection*) self->priv->transcoders, (RygelTranscoder*) (_tmp8_ = rygel_mp2_ts_transcoder_new (RYGEL_MP2_TS_PROFILE_HD)));
+			_g_object_unref0 (_tmp8_);
 		}
 	}
-	(config == NULL) ? NULL : (config = (g_object_unref (config), NULL));
+	_g_object_unref0 (config);
 	return self;
 }
 
 
-RygelTranscodeManager* rygel_transcode_manager_new (void) {
-	return rygel_transcode_manager_construct (RYGEL_TYPE_TRANSCODE_MANAGER);
-}
-
-
-static char* rygel_transcode_manager_real_create_uri_for_item (RygelTranscodeManager* self, RygelMediaItem* item, const char* transcode_target, char** protocol) {
+static char* rygel_transcode_manager_real_create_uri_for_item (RygelTranscodeManager* self, RygelMediaItem* item, gint thumbnail_index, const char* transcode_target, char** protocol) {
 	g_return_val_if_fail (self != NULL, NULL);
 	g_critical ("Type `%s' does not implement abstract method `rygel_transcode_manager_create_uri_for_item'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
 	return NULL;
 }
 
 
-char* rygel_transcode_manager_create_uri_for_item (RygelTranscodeManager* self, RygelMediaItem* item, const char* transcode_target, char** protocol) {
-	return RYGEL_TRANSCODE_MANAGER_GET_CLASS (self)->create_uri_for_item (self, item, transcode_target, protocol);
+char* rygel_transcode_manager_create_uri_for_item (RygelTranscodeManager* self, RygelMediaItem* item, gint thumbnail_index, const char* transcode_target, char** protocol) {
+	return RYGEL_TRANSCODE_MANAGER_GET_CLASS (self)->create_uri_for_item (self, item, thumbnail_index, transcode_target, protocol);
 }
 
 
-static void rygel_transcode_manager_real_add_resources (RygelTranscodeManager* self, GeeArrayList* resources, RygelMediaItem* item, GError** error) {
+static gpointer _g_object_ref0 (gpointer self) {
+	return self ? g_object_ref (self) : NULL;
+}
+
+
+static gint _rygel_media_item_compare_transcoders_gcompare_data_func (void* a, void* b, gpointer self) {
+	return rygel_media_item_compare_transcoders (self, a, b);
+}
+
+
+static void _g_list_free_g_object_unref (GList* self) {
+	g_list_foreach (self, (GFunc) g_object_unref, NULL);
+	g_list_free (self);
+}
+
+
+static void rygel_transcode_manager_real_add_resources (RygelTranscodeManager* self, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, GError** error) {
 	GError * _inner_error_;
+	GList* list;
 	g_return_if_fail (self != NULL);
-	g_return_if_fail (resources != NULL);
+	g_return_if_fail (didl_item != NULL);
 	g_return_if_fail (item != NULL);
 	_inner_error_ = NULL;
-	if (g_str_has_prefix (item->upnp_class, RYGEL_MEDIA_ITEM_IMAGE_CLASS)) {
-		/* No  transcoding for images yet :(*/
-		return;
-	}
-	/* First add resource of the transcoders that are primarily meant for
-	 the UPnP class of the item concerned*/
+	list = NULL;
 	{
 		GeeIterator* _transcoder_it;
-		/* First add resource of the transcoders that are primarily meant for
-		 the UPnP class of the item concerned*/
-		_transcoder_it = gee_iterable_iterator ((GeeIterable*) self->priv->transcoders);
-		/* First add resource of the transcoders that are primarily meant for
-		 the UPnP class of the item concerned*/
-		while (gee_iterator_next (_transcoder_it)) {
+		_transcoder_it = gee_abstract_collection_iterator ((GeeAbstractCollection*) self->priv->transcoders);
+		while (TRUE) {
 			RygelTranscoder* transcoder;
-			/* First add resource of the transcoders that are primarily meant for
-			 the UPnP class of the item concerned*/
-			transcoder = (RygelTranscoder*) gee_iterator_get (_transcoder_it);
-			if (g_str_has_prefix (item->upnp_class, rygel_transcoder_get_upnp_class (transcoder))) {
-				rygel_transcoder_add_resources (transcoder, resources, item, self, &_inner_error_);
-				if (_inner_error_ != NULL) {
-					g_propagate_error (error, _inner_error_);
-					(transcoder == NULL) ? NULL : (transcoder = (g_object_unref (transcoder), NULL));
-					(_transcoder_it == NULL) ? NULL : (_transcoder_it = (g_object_unref (_transcoder_it), NULL));
-					return;
-				}
+			if (!gee_iterator_next (_transcoder_it)) {
+				break;
 			}
-			(transcoder == NULL) ? NULL : (transcoder = (g_object_unref (transcoder), NULL));
+			transcoder = (RygelTranscoder*) gee_iterator_get (_transcoder_it);
+			if (rygel_transcoder_get_distance (transcoder, item) != G_MAXUINT) {
+				list = g_list_append (list, _g_object_ref0 (transcoder));
+			}
+			_g_object_unref0 (transcoder);
 		}
-		(_transcoder_it == NULL) ? NULL : (_transcoder_it = (g_object_unref (_transcoder_it), NULL));
+		_g_object_unref0 (_transcoder_it);
 	}
-	/* Then add resources from other transcoders*/
+	list = g_list_sort_with_data (list, _rygel_media_item_compare_transcoders_gcompare_data_func, item);
 	{
-		GeeIterator* _transcoder_it;
-		/* Then add resources from other transcoders*/
-		_transcoder_it = gee_iterable_iterator ((GeeIterable*) self->priv->transcoders);
-		/* Then add resources from other transcoders*/
-		while (gee_iterator_next (_transcoder_it)) {
+		GList* transcoder_collection;
+		GList* transcoder_it;
+		transcoder_collection = list;
+		for (transcoder_it = transcoder_collection; transcoder_it != NULL; transcoder_it = transcoder_it->next) {
 			RygelTranscoder* transcoder;
-			/* Then add resources from other transcoders*/
-			transcoder = (RygelTranscoder*) gee_iterator_get (_transcoder_it);
-			if (!g_str_has_prefix (item->upnp_class, rygel_transcoder_get_upnp_class (transcoder))) {
-				rygel_transcoder_add_resources (transcoder, resources, item, self, &_inner_error_);
+			transcoder = _g_object_ref0 ((RygelTranscoder*) transcoder_it->data);
+			{
+				GUPnPDIDLLiteResource* _tmp0_;
+				_tmp0_ = rygel_transcoder_add_resource (transcoder, didl_item, item, self, &_inner_error_);
 				if (_inner_error_ != NULL) {
 					g_propagate_error (error, _inner_error_);
-					(transcoder == NULL) ? NULL : (transcoder = (g_object_unref (transcoder), NULL));
-					(_transcoder_it == NULL) ? NULL : (_transcoder_it = (g_object_unref (_transcoder_it), NULL));
+					_g_object_unref0 (transcoder);
+					__g_list_free_g_object_unref0 (list);
 					return;
 				}
+				_g_object_unref0 (_tmp0_);
+				_g_object_unref0 (transcoder);
 			}
-			(transcoder == NULL) ? NULL : (transcoder = (g_object_unref (transcoder), NULL));
 		}
-		(_transcoder_it == NULL) ? NULL : (_transcoder_it = (g_object_unref (_transcoder_it), NULL));
 	}
+	__g_list_free_g_object_unref0 (list);
 }
 
 
-void rygel_transcode_manager_add_resources (RygelTranscodeManager* self, GeeArrayList* resources, RygelMediaItem* item, GError** error) {
-	RYGEL_TRANSCODE_MANAGER_GET_CLASS (self)->add_resources (self, resources, item, error);
+void rygel_transcode_manager_add_resources (RygelTranscodeManager* self, GUPnPDIDLLiteItem* didl_item, RygelMediaItem* item, GError** error) {
+	RYGEL_TRANSCODE_MANAGER_GET_CLASS (self)->add_resources (self, didl_item, item, error);
 }
 
 
 RygelTranscoder* rygel_transcode_manager_get_transcoder (RygelTranscodeManager* self, const char* target, GError** error) {
+	RygelTranscoder* result;
 	GError * _inner_error_;
 	RygelTranscoder* transcoder;
 	g_return_val_if_fail (self != NULL, NULL);
@@ -398,30 +393,87 @@ RygelTranscoder* rygel_transcode_manager_get_transcoder (RygelTranscodeManager*
 	transcoder = NULL;
 	{
 		GeeIterator* _iter_it;
-		_iter_it = gee_iterable_iterator ((GeeIterable*) self->priv->transcoders);
-		while (gee_iterator_next (_iter_it)) {
+		_iter_it = gee_abstract_collection_iterator ((GeeAbstractCollection*) self->priv->transcoders);
+		while (TRUE) {
 			RygelTranscoder* iter;
+			if (!gee_iterator_next (_iter_it)) {
+				break;
+			}
 			iter = (RygelTranscoder*) gee_iterator_get (_iter_it);
 			if (rygel_transcoder_can_handle (iter, target)) {
-				RygelTranscoder* _tmp1_;
 				RygelTranscoder* _tmp0_;
-				_tmp1_ = NULL;
-				_tmp0_ = NULL;
-				transcoder = (_tmp1_ = (_tmp0_ = iter, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (transcoder == NULL) ? NULL : (transcoder = (g_object_unref (transcoder), NULL)), _tmp1_);
+				transcoder = (_tmp0_ = _g_object_ref0 (iter), _g_object_unref0 (transcoder), _tmp0_);
 			}
-			(iter == NULL) ? NULL : (iter = (g_object_unref (iter), NULL));
+			_g_object_unref0 (iter);
 		}
-		(_iter_it == NULL) ? NULL : (_iter_it = (g_object_unref (_iter_it), NULL));
+		_g_object_unref0 (_iter_it);
 	}
 	if (transcoder == NULL) {
 		_inner_error_ = g_error_new (RYGEL_HTTP_REQUEST_ERROR, RYGEL_HTTP_REQUEST_ERROR_NOT_FOUND, "No transcoder available for target format '%s'", target);
 		if (_inner_error_ != NULL) {
 			g_propagate_error (error, _inner_error_);
-			(transcoder == NULL) ? NULL : (transcoder = (g_object_unref (transcoder), NULL));
+			_g_object_unref0 (transcoder);
 			return NULL;
 		}
 	}
-	return transcoder;
+	result = transcoder;
+	return result;
+}
+
+
+static char* rygel_transcode_manager_real_get_protocol (RygelTranscodeManager* self) {
+	g_return_val_if_fail (self != NULL, NULL);
+	g_critical ("Type `%s' does not implement abstract method `rygel_transcode_manager_get_protocol'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
+	return NULL;
+}
+
+
+char* rygel_transcode_manager_get_protocol (RygelTranscodeManager* self) {
+	return RYGEL_TRANSCODE_MANAGER_GET_CLASS (self)->get_protocol (self);
+}
+
+
+static char* rygel_transcode_manager_real_get_protocol_info (RygelTranscodeManager* self) {
+	char* result;
+	char* protocol_info;
+	g_return_val_if_fail (self != NULL, NULL);
+	protocol_info = g_strdup ("");
+	{
+		GeeIterator* _transcoder_it;
+		_transcoder_it = gee_abstract_collection_iterator ((GeeAbstractCollection*) self->priv->transcoders);
+		while (TRUE) {
+			RygelTranscoder* transcoder;
+			char* _tmp6_;
+			char* _tmp5_;
+			char* _tmp4_;
+			char* _tmp3_;
+			char* _tmp2_;
+			char* _tmp1_;
+			if (!gee_iterator_next (_transcoder_it)) {
+				break;
+			}
+			transcoder = (RygelTranscoder*) gee_iterator_get (_transcoder_it);
+			if (_vala_strcmp0 (protocol_info, "") != 0) {
+				char* _tmp0_;
+				protocol_info = (_tmp0_ = g_strconcat (protocol_info, ",", NULL), _g_free0 (protocol_info), _tmp0_);
+			}
+			protocol_info = (_tmp6_ = g_strconcat (protocol_info, _tmp5_ = g_strconcat (_tmp4_ = g_strconcat (_tmp3_ = g_strconcat (_tmp2_ = g_strconcat (_tmp1_ = rygel_transcode_manager_get_protocol (self), ":*:", NULL), rygel_transcoder_get_mime_type (transcoder), NULL), ":DLNA.ORG_PN=", NULL), rygel_transcoder_get_dlna_profile (transcoder), NULL), NULL), _g_free0 (protocol_info), _tmp6_);
+			_g_free0 (_tmp5_);
+			_g_free0 (_tmp4_);
+			_g_free0 (_tmp3_);
+			_g_free0 (_tmp2_);
+			_g_free0 (_tmp1_);
+			_g_object_unref0 (transcoder);
+		}
+		_g_object_unref0 (_transcoder_it);
+	}
+	result = protocol_info;
+	return result;
+}
+
+
+char* rygel_transcode_manager_get_protocol_info (RygelTranscodeManager* self) {
+	return RYGEL_TRANSCODE_MANAGER_GET_CLASS (self)->get_protocol_info (self);
 }
 
 
@@ -430,6 +482,8 @@ static void rygel_transcode_manager_class_init (RygelTranscodeManagerClass * kla
 	g_type_class_add_private (klass, sizeof (RygelTranscodeManagerPrivate));
 	RYGEL_TRANSCODE_MANAGER_CLASS (klass)->create_uri_for_item = rygel_transcode_manager_real_create_uri_for_item;
 	RYGEL_TRANSCODE_MANAGER_CLASS (klass)->add_resources = rygel_transcode_manager_real_add_resources;
+	RYGEL_TRANSCODE_MANAGER_CLASS (klass)->get_protocol = rygel_transcode_manager_real_get_protocol;
+	RYGEL_TRANSCODE_MANAGER_CLASS (klass)->get_protocol_info = rygel_transcode_manager_real_get_protocol_info;
 	G_OBJECT_CLASS (klass)->finalize = rygel_transcode_manager_finalize;
 }
 
@@ -442,7 +496,7 @@ static void rygel_transcode_manager_instance_init (RygelTranscodeManager * self)
 static void rygel_transcode_manager_finalize (GObject* obj) {
 	RygelTranscodeManager * self;
 	self = RYGEL_TRANSCODE_MANAGER (obj);
-	(self->priv->transcoders == NULL) ? NULL : (self->priv->transcoders = (g_object_unref (self->priv->transcoders), NULL));
+	_g_object_unref0 (self->priv->transcoders);
 	G_OBJECT_CLASS (rygel_transcode_manager_parent_class)->finalize (obj);
 }
 
@@ -457,5 +511,16 @@ GType rygel_transcode_manager_get_type (void) {
 }
 
 
+static int _vala_strcmp0 (const char * str1, const char * str2) {
+	if (str1 == NULL) {
+		return -(str1 != str2);
+	}
+	if (str2 == NULL) {
+		return str1 != str2;
+	}
+	return strcmp (str1, str2);
+}
+
+
 
 
